;;; (C)2019 Board of Trustees of the University of Illinois

;;; Simple hydrological model for field experiments for CNH-3
;;; based on WULUM (Zellner, 2007), and L-GrID (Zellner et al., in review)
;;; Authors: Moira Zellner, Dean Massey, Federico Bert, and Santiago Rovere
;;; Do not cite or disseminate without authors' permission

;;; Please note, in reference to variables:
;;; Absolute means that the value for the variable is in reference to seal level. E.g. an elevation of -1.0 means 1 foot below sea level
;;; Relative means that the value for the variable is sin relation to some other value. E.g. capillary-soil-column value of 0.8 means the distance from the top to the bottom of
;;; capillary zone is 0.8 m.

globals [
  
  crop-list ;; A list of all of the types of crops. Used to update variables when procedure would be identical but for the crop name.  
 
  ;; Time
  days-in-year               ;;; The number of days in the year - needed for proper cycling with leap years
  day                        ;;; The current day of the year. Gets reset at the end of a year
  year                       ;;; The current year
 ; divtimesteps               ;;; how many times the flow happens per day. Assuming we will always have this be once per day, so commenting out
  
  ;; Spatial
  surface-cell               ;;; The area of a patch (m^2)
  Landscape-Elevation-list   ;;; List of the all the patches ordered by elevation (small to large). Used in surface flow
  landscape-center-patch               ;;; This records the patch that is at the center of the landscape
 
  ;; Daily values
  ;;;;;;;;;;;;;;;
 
  daily-rain-amount          ;;; Daily precipitation in meters.
  daily-ETo                  ;;; Rerefence evapotranspiration - in m. Based on climate and a reference crop.
  
  ;;; Daily Kcb values for each crop. Unitless
  daily-Kcb-soy                
  daily-kcb-pasture            
  daily-Kcb-maize
  daily-kcb-cover-before-soy
  daily-kcb-wheat-before-soy
  daily-kcb-short-cycle-soy

  ;; Daily root length values foe each crop. In meters (relative)
  daily-soy-root-length      
  daily-pasture-root-length  
  daily-maize-root-length
  daily-cover-before-soy-root-length
  daily-wheat-before-soy-root-length
  daily-short-cycle-soy-root-length
  
  ;; Daily potential root regrowth values for each crop. In meters (relative)
  ;; Calculated before, but the potential is  today's length minus the previous length. For pasture, it is 1% of fthe maximum length
  daily-potential-root-regrowth-soy
  daily-potential-root-regrowth-pasture
  daily-potential-root-regrowth-maize
  daily-potential-root-regrowth-cover-before-soy
  daily-potential-root-regrowth-wheat-before-soy
  daily-potential-root-regrowth-short-cycle-soy
  
  ;; Crop cycling and yield-related values
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  ;; During the individual adaptation scenario, this is the day farmers check the water table to see if one of these crops is planted. The other crops plant on day 0.
  day-check-for-soy 
  day-check-for-maize 
  day-check-for-short-cycle-soy 
  
  ;; The day each type of crop harvests
  day-harvest-pasture
  day-harvest-soy
  day-harvest-maize
  day-harvest-cover-before-soy
  day-harvest-wheat-before-soy
  day-harvest-short-cycle-soy 
  
  ;; Temporary variable
  critical-now? ;; Is a particular crop in a critical growth stage now? true/false? This is really just a temporary local variable, but it needs to be introduced here otherwise the "run" primitive won't work with it
    
  ;; The rest grouped by crop, since the variables repeat for each crop
  
  ;; Soy
  critical-grow-soy-start
  critical-grow-soy-end
  critical-growing-period-soy?
  soy-tonnage-per-hectare
  soy-tonnage-per-patch
  soy-maximum-output
  total-soy-output
  
  ;; Pasture
  critical-growing-period-pasture?
  pasture-tonnage-per-hectare
  pasture-tonnage-per-patch
  pasture-maximum-output
  total-pasture-output
  
  ;; Maize
  critical-grow-maize-start
  critical-grow-maize-end
  critical-growing-period-maize?
  maize-tonnage-per-hectare
  maize-tonnage-per-patch
  maize-maximum-output
  total-maize-output
  
  ;; Cover crop (before soy)
  critical-grow-cover-before-soy-start
  critical-grow-cover-before-soy-end
  critical-growing-period-cover-before-soy?
  cover-before-soy-tonnage-per-hectare
  cover-before-soy-tonnage-per-patch
  cover-before-soy-maximum-output
  total-cover-before-soy-output
  
  ;; Wheat (before soy)
  critical-grow-wheat-before-soy-start
  critical-grow-wheat-before-soy-end
  critical-growing-period-wheat-before-soy?
  wheat-before-soy-tonnage-per-hectare
  wheat-before-soy-tonnage-per-patch
  wheat-before-soy-maximum-output
  total-wheat-before-soy-output
  
  ;; Short cycle soy
  critical-grow-short-cycle-soy-start
  critical-grow-short-cycle-soy-end
  critical-growing-period-short-cycle-soy?
  short-cycle-soy-tonnage-per-hectare
  short-cycle-soy-tonnage-per-patch
  short-cycle-soy-maximum-output
  total-short-cycle-soy-output
  
  ;; Importing lists
  ;;;;;;;;;;;;;;;;;;
    
  current-year-values        ;; A list of all the daily values for only the current year. If using multi-year data, the model cuts the current year values from the larger list.
  
  ;; Lists of daily values for each land cover. The lists have KCB, root length and potential root regrowth, in that order.
  soy-values
  pasture-values
  maize-values
  cover-before-soy-values
  wheat-before-soy-values
  short-cycle-soy-values  
  
  imported-values-list       ;; Each line is a separate day and values listed in this order:
                             ;; precipitation, ETo, kc soy, kc pasture, Kc Cover crop, roots soy, roots pasture, roots cover crop (8 items total)   
  imported-landscape-list    ;; Values for each patch in the landscape, one row per patch, columns in the following order: pxcor pycor farmId elevation initialLandUse
  
  ;; Water tracking and errors
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  ;; Mass balance
  ;; These heights in the below variables are in patch terms - multiply by patch surface area to convert to volume
  Landscape-cumulative-water-height ;;; Tracks the total amount of water after setup in the (soil or surface) or later entered the landscape (rain).
  Landscape-current-water-height     ;;; Height of water in the landscape now (soil and surface)
  Total-loss-through-ET      ;;; Height of water lost through evapotranspiration
  Total-loss-through-runoff  ;;; Height of water that left the landscape due to runoff
  Landscape-cumulative-net-boundary-gain ;;; Water that enters due to the boundary conditions
  Landscape-cumulative-net-boundary-loss
  Iteration-discrepancy          ;;; aka a margin of error; % difference between water created and what exists now. Negative value means water loss than there should be - some water unaccounted for
  Cumulative-discrepancy
  
  ;; Variables used to debug any errors
  error-code                 ;;; used to display information about the potential source of error
  error-code-update-RZ-UZ
 
  ;; Used by graphs
  sum-E                      ;;; Sum of the day's evaporation from all patches
  sum-T                      ;;; Sum of the cay's transpiration for all patches
]

patches-own [
   
  ;; General
  farmId     ;;; Not used in any meaningful way now by any processes, but this will be an identifier for farms that cover more than one patch.    
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; Water and soil horizons ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  elevation                 ;; The elevation of the patch in meters. Absolute.

  large-pore-specific-yield ;; The specific yield of large pores, or the % of the soil column that is voids for large pores
  soil-bottom               ;; Maximum soil depth in meters. For purposes of this model, this the lowest point possible in the landscape. Absolute.
  Neighbor-Elevation-list   ;; A list of the patch's neighbors sorted by elevation (low to high) and used by the surface flow procedure.
  boundary-cell?            ;; If we are assuming that this landscape is connected to the groundwater beyond the landscape, is this patch on the edge of the landscape? Yes/no.
  patch-iteration-boundary-gain  ;; Used if this is a boundary cell to account for water that ENTERS the landscape via connection to the external groundwater
  patch-iteration-boundary-loss
  
  ;; Surface water
  surface-water-column      ;; Height of water on the surface of a cell in meters. Relative.
  initial-surface-water-column
  
  ;; Groundwater zone
  groundwater-top           ;; Indicates the top of the groundwater zone in meters. Absolute.
  groundwater-top-FUTURE
  initial-groundwater-top
  groundwater-water-column  ;; Height of the water column in the groundwater zone in meters. Based on the soil column height and specific yield. Relative.
  initial-groundwater-water-column
  groundwater-water-column-FUTURE
  groundwater-water-volume  ;; Volume of the water in the groundwater in cubic meters.

  ;;; Capillary Zone
  capillary-top             ;; Height of the top of the capillary zone in meters. Absolute.
  capillary-top-FUTURE
    
  ;;; Upper zone aka non-water table influenced Zone
  upper-zone-water-column   ;; Height of the water column in the upper zone in meters. Based on soil column and medium pore specific yield. Relative.
  
  ;; Root zone
  root-zone-bottom          ;; The location of the bottom of the root zone in meters. Absolute.
  root-zone-bottom-FUTURE
  root-zone-water-column    ;; Height of the water column in the root zone in meters. Based on soil column and medium pore specific yield. Relative.
  
    
  RZ-UZ-over                ;; Used in the water accounting to check for an error when the root zone or upper zone water columns go above 100% saturation levels.
  
  ;; Both capillary and root zones
  water-available-root-zone ;; Height of the water column available for transpiration in the root zone in meters. Relative.
  water-available-capillary-zone  ;; Height of the water column available for transpiration in the capillary zone in meters. Relative.
    
  ;; capillary, upper, and root zones
  patch-total-water-height        ;; The total height of water in the groundwater, upper, and root zones and on the surface. The height is in patch terms - multiply by patch surface area to convert to volume
  
  ;; Previous zone heights used when updating water accounting when the zones change
;  Previous-root-zone-soil-column
;  Previous-upper-zone-soil-column
    
  ;;;;;;;;;;;
  ;; Crops ;;
  ;;;;;;;;;;;
  
  ;; Land cover
  land-cover                ;; The current land cover for the patch - fallow or a particular crop.
  last-day-0-decision       ;; The planting decision that was made on day 0 of the year.
  planting-history          ;; A list of the planting history of this patch.
  stressed?                 ;; Has the plant ever received less than 100% of its daily need? Yes/no.
  dead?                     ;; Is the plant dead? Plant death can happen due to unmet water demand or harvesting of crops.
  days-no-water             ;; The number of consecutive days the plant has had 100% unmet need of ETC (ETc) for transpiration.
  day-harvest               ;; The day of the year the current crop will harvest.

  day-check-WTD             ;; In individual adaptation scenario, the day the patch will make another planting decision based on water table depth.
  Rotation-year             ;; Used to record the place in a predetermined order of plantings in certain planting scenarios
    
  ;; Roots
  roots-damaged?            ;; Indicates if the crop's roots have ever reached into the groundwater. Yes/no.
  root-length               ;; Length of the roots, will vary with land cover and time of the year(m). Relative.
  root-bottom               ;; Location of the bottom of the roots in meters. Absolute.
  percent-roots-in-root-zone      ;; % of root length in the root zone.
  percent-roots-in-capillary-zone ;; % of root length in the capillary zone.
  root-length-root-zone     ;; Length of the roots in the root zone in meters. Relative.
  root-length-capillary-zone ;; Length of the roots in the capillary zone in meters. Relative.
  daily-root-value          ;; Generic variable used to update the daily root length for this patch based on its land cover. This is really a local variable,
                            ;; but in order for it to be set using a "run" procedure, it needs to be predefined.
  daily-root-regrowth-value ;; Generic variable used to update the daily root regrowth, for this patch based on its land cover. See previous variable.

  ;; Evapotranspiration
  ETc                ;; Height of water column that the crop needs to consume through evapotranspiration needed per unit time. Vary with land cover and time of year. In meters. Relative. Based on ETo (reference evapotranspiration) and Kcb
  water-deficit             ;; Height of the water column of unmet ETc for transpiration for the current day. In meters. Relative.
  transpiration-amount      ;; Amount of water that the plant actually transpired. Based on ETc and water available. In meters. Relative.
  evaporation-amount        ;; The actual amount that will/has evaporated in meters. Relative. Based on reference evapotranspiration and what has evaporated already that day already through plant leaves.
  
  ;; Crop yield
  daily-unmet-need          ;; The daily value for the % unmet ETc needed for transpiration.
  cumulative-percent-daily-unmet-need ;; The cumulative % unmet demand of days this cycle when the crop was in a critical growth period.
  crop-yield-factor         ;; Crop yield factor is relates directly to unmet water demand during critical growth periods and used to determine yields.
                            ;; The factor is scaled from 0 (full potential output) to 1 (no output) 
  annual-yield              ;; The yield for the crop that just harvested on this patch. 
  days-current-crop-critical      ;; The number of days the current crop has been in a critical growth period.

  ;; Cumulative years each type of crop planted  
  years-soy-planted
  years-pasture-planted
  years-maize-planted
  years-cover-before-soy-planted
  years-wheat-before-soy-planted
  years-short-cycle-soy-planted
  
  ;; Cumulative yield for each crop type, in tons.
  patch-soy-output
  patch-pasture-output
  patch-maize-output
  patch-cover-before-soy-output
  patch-wheat-before-soy-output
  patch-short-cycle-soy-output
]

to setup
  ;;setup landscape and hydrological variables
  
  clear-all ;; clear-output; wipe the command center
  
  ;;; Setting a list of all the crops that will be used for more compact updating of values when subprocedures repeat for each crop
  set crop-list ( sentence "pasture" "maize" "soy" "cover-before-soy" "wheat-before-soy" "short-cycle-soy" )
  
  ;;; imports the lists of daily values relating to rain and ETo independent of land cover; cover specific daily values; and the landscape file
  import-values
    
  ;; Creates a list of the rain and ETo values only for the upcoming year
  setup-annual-list
  ;; Determine when crops plant and harvest, also critical need periods
  setup-growing-cycles
  
  ;; Initialize time variables
  set day 0
  set year 0
  ;  set unit-time 1 ;; 1 day ;;;; unit time is not used by any other processes, so is it needed?
 ; set divtimesteps 1 ;;; For groundwater flow, how many times the flow happens per day
  
  ;; Landscape size
  change-world-dimensions ;; Sizes the landscape to match a) input file dimensions or b) sliders regarding dimensions
  set surface-cell Patch-dimensions * Patch-dimensions ;;; The area of a cell in square meters. Patch dimensions is set by a slider. E.g. 100 m x 100 m = 1 hectare ; 10,000 m^2
  set landscape-center-patch find-center-patch ;; Report the patch at the center of the landscape - should be patch 0 0. This is used by various processes
  
  
  ;; Elevation and soil
  setup-elevation ;; Sets the elevation of the landscape, either a) from file or b) generic slop by slider; creates list of all patches and all patches' neighbors
  setup-soil ;; Sets specific yield and the depth of the soil column

  ;;; Sets up boundary conditions, groundwater/surface water, and the capillary zone
  setup-initial-groundwater   ;; Setup of bounrary conditions. initializes the groundwater levels, any initial surface water resulting from the initial groundwater water level, and the capillary zone, which is directly dependent on the groundwater levels
  
  setup-upper-and-root-zones ;; Sets up the soil and water columns for the root and upper zones
  setup-farms-crops ;; Sets up crop and yield variables
  setup-crop-yield ;; Sets values related to critical growing periods and yield for each crop
  
  ;;; The model needs to keep track of the net water balance and the margin of error for how moch water there started in the landscape, how much entered, and how much left compared to all the water there now
  ;;s The following sets up the total water created by the initialization
  ask patches [
    set patch-total-water-height precision ( groundwater-water-column + surface-water-column + upper-zone-water-column + root-zone-water-column + ( (capillary-top - groundwater-top ) * medium-pore-specific-yield ) ) 4
  ]
  set Landscape-current-water-height precision ( sum [ patch-total-water-height ] of patches ) 4
  set Landscape-cumulative-water-height ( sum [ patch-total-water-height ] of patches )
  
  if print-total-water-heights? [
    print "SETUP"
    ask patch 0 0 [
      type "self = " print self
      type "surface-water-column = " print surface-water-column
      type "root-zone-water-column = " print root-zone-water-column
      type "upper-zone-water-column = " print upper-zone-water-column
      type "CZ-water-column = " print ( (capillary-top - groundwater-top ) * medium-pore-specific-yield )
      type "groundwater-water-column = " print groundwater-water-column
      type "patch-total-water-height = " print patch-total-water-height
      type "root-zone-bottom = " print root-zone-bottom
      type "capillary-top = " print capillary-top
      type "groundwater-top = " print groundwater-top
    ]
    type "landscape-total-water-height = " print Landscape-current-water-height
  ]
  
  apply-patch-labels ;; This has the patches label themselves with a variable's value indicated by a chooser
  apply-colors ;; This has the patches color themselves based on land cover and the crop's status and produces an icon if there is surface water
  
  ;; Setup the export file if the option to export patch level data to a separate file during a run is activated
  if Export-data? = true [
    setup-export
  ]
  reset-ticks 
end

to import-values
  ;;; imports the lists of daily values relating to rain and ETo independent of land cover; cover specific daily values; and the landscape file
  ;;; Saves these all in separate lists of values
  
  ;;; Climate: rain, ETo, and year
  if Water-Scenario = "Dry" [
    file-open "dry-year-revised.txt"
    set imported-values-list file-read
    file-close
  ]
  
  if Water-Scenario = "Average" [
    file-open "average-year-revised.txt"
    set imported-values-list file-read
    file-close
  ] 
  
  if Water-Scenario = "Wet" [
    file-open "wet-year-revised.txt"
    set imported-values-list file-read
    file-close
  ]
  
  if Water-Scenario = "Multi-year Pampas data" [
    file-open "multi-year-revised.txt"
    set imported-values-list file-read
    file-close   
  ]
  
  if Water-Scenario = "5-year-series" [
    file-open "5-year-series.txt"
    set imported-values-list file-read
    file-close   
  ]
  
  ;;; Crops: ETc, root length, and potential regrowth
  file-open "soy-values.txt"
  set soy-values file-read
  file-close  
  
  file-open "pasture-values.txt"
  set pasture-values file-read
  file-close  
  
  file-open "maize-values.txt"
  set maize-values file-read
  file-close  
  
  file-open "cover-before-soy-values.txt"
  set cover-before-soy-values file-read
  file-close  
  
  file-open "wheat-before-soy-values.txt"
  set wheat-before-soy-values file-read
  file-close  
  
  file-open "short-cycle-soy-values.txt"
  set short-cycle-soy-values file-read
  file-close  
    
  ;; Landscape file
  if Import-landscape? [
    ;; pxcor pycor farmId elevation initialLandUse
    file-open "Patch.txt"
    set imported-landscape-list file-read
    file-close
  ]
end

to setup-annual-list
;; Creates a list of the rain and ETo values only for the upcoming year
  
  ifelse Water-Scenario = "Dry" or Water-Scenario = "Average" or Water-Scenario = "Wet"[
    ;; Days in the year will always be 365 (starting from 0, then 364) and the values corresponds exactly to one of the input files already
    set current-year-values imported-values-list
    set days-in-year 364 ;; count starting at 0. This will not change during a run.
  ]
  [
    ;; Days in year will vary, also need to cut up the larger input file for just this year's values
    let year-column [] ;; this is used to determine the number of days in a year
    let field-counter 0 ;; tracks the field of the file that is being examined
    ;; Looks at the entire list and puts all the year values in one file
    foreach imported-values-list [
      if ( field-counter - 2 ) mod 3 = 0 [ ;; the third column in the input sheet meets this criteria
        set year-column lput ? year-column 
      ]
      set field-counter field-counter + 1
    ]
    let current-year-only filter [? = 0] year-column ;; Filter out everything except year 0
    set days-in-year length current-year-only ;; the days in the year is equal to the number of entries in the file now
    ;; This cuts up the larger input file into the upcoming year values
    set current-year-values sublist imported-values-list 0 ( 3 * days-in-year ) ;; 3 columns in the input file. 3 x days = 1 beyond the number, which is needed since sublist requires going one beyond. It is one beyond already because the lists start at 0 not 1    
    set days-in-year days-in-year - 1 ;; count starting at 0, so subtract 1
  ]
end

to setup-growing-cycles
  ;; Determine when crops plant and harvest, also critical need periods
  ;; This would be difficult to condense and make one procedure for all crop types, since some crops plant immediately at the start of the year and some harvest at the very end of the year
  ;; and there are different rules about critical growing for each crop
  
  ;;;;;;;;;;;;;;;;;;;
  ;;; REGULAR SOY ;;;
  ;;;;;;;;;;;;;;;;;;;
  ;;;
  ;;; creating a list of this year's KC soy values
  ;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  let annual-KC-soy-values []
  let counter-soy-values 0
  
  ;; this is not the ideal way to do it, but item or position are not independent variables that can be called, e.g. can't say if position mod 4 = 0 [...
  ;; instead need to go through the list and take the item from every row in the input file this way.
  foreach soy-values [
    if counter-soy-values mod 3 = 0 [
      set annual-KC-soy-values lput ? annual-KC-soy-values
    ]
    set counter-soy-values counter-soy-values + 1
  ]
  ;;;;;;
  ;; Determine when the max KC is and make the critical growing period +- 30 days
  ;;;;;;
  
  let max-KC-soy max annual-KC-soy-values
;  show max-KC-soy
  let day-max-kc-soy position max-KC-soy annual-KC-soy-values
  set critical-grow-soy-start day-max-kc-soy - 30
  set critical-grow-soy-end day-max-kc-soy + 31 ;; one day beyond 30, so ensure +- 30 days

  ;;;;;;
  ;; Determine when soy starts growing and when check WT depth to see if soy will be planted
  ;; To do this, mark what day the KC for soy first is > 0
  ;;;;;;
  
  let counter-soy-KC 0
  let previous-kcsoy-here 0
  let day-plant-soy 0
  set day-harvest-soy 0
  
  while [ day-plant-soy = 0 or day-harvest-soy = 0 ] [
    let kc-value item counter-soy-KC annual-KC-soy-values ;; record the KC value corresponding to the day from the counter
    if kc-value > 0 and previous-kcsoy-here = 0 [ ;; previous was 0 but now it's above 0, growth started
      set day-plant-soy counter-soy-KC ;; this is the day growth started
    ]
    if kc-value = 0 and previous-kcsoy-here > 0 [ ;; current value 0 but previous was above 0
      set day-harvest-soy counter-soy-KC - 1 ;; the previous day was the day to harvest
    ] 
    set previous-kcsoy-here kc-value
    set counter-soy-KC counter-soy-KC + 1
  ]
    
  set day-check-for-soy day-plant-soy - 7 ;; check  7 days before the crop will start growing
  
  ;;;  7 days is a placeholder for now, we can always change it later.
  
  ;;;;;;;;;;;;;
  ;;; MAIZE ;;;
  ;;;;;;;;;;;;;
  ;;;
  ;;; creating a list of this year's KC maize values
  ;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  let annual-KC-maize-values []
  let counter-maize-values 0
  foreach maize-values [
    if counter-maize-values mod 3 = 0 [
      set annual-KC-maize-values lput ? annual-KC-maize-values
    ]
    set counter-maize-values counter-maize-values + 1
  ]
  ;;;;;;
  ;; Determine when the max KC is and make the critical growing period +- 30 days
  ;;;;;;  
  let max-KC-maize max annual-KC-maize-values
  let day-max-kc-maize position max-KC-maize annual-KC-maize-values
  set critical-grow-maize-start day-max-kc-maize - 30
  set critical-grow-maize-end day-max-kc-maize + 31 ;; one day beyond 30, so ensure +- 30 days

  ;;;;;;
  ;; Determine when maize starts growing, when to check WT depth, and when harvst happens
  ;;;;;;
  
  let counter-maize-KC 0
  let previous-kcmaize-here 0
  let day-plant-maize 0
  set day-harvest-maize 0
  
  while [ day-plant-maize = 0 or day-harvest-maize = 0 ] [
    let kc-value item counter-maize-KC annual-KC-maize-values ;; record the KC value corresponding to the day from the counter
    if kc-value > 0 and previous-kcmaize-here = 0 [
      set day-plant-maize counter-maize-KC ;; this is the year growth started
    ]
    ;;
    if kc-value = 0 and previous-kcmaize-here > 0 [ ;; current value 0 but previous was above 0
      set day-harvest-maize counter-maize-KC - 1 ;; the previous day was the day to harvest
    ] 
    set previous-kcmaize-here kc-value
    set counter-maize-KC counter-maize-KC + 1
  ]
    
  set day-check-for-maize day-plant-maize - 7 ;; check  7 days before the crop will start growing
  
  ;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; COVER BEFORE SOY ;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;
  ;;; creating a list of this year's KC cover-before-soy values
  ;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  let annual-KC-cover-before-soy-values []
  let counter-cover-before-soy-values 0
  foreach cover-before-soy-values [
    if counter-cover-before-soy-values mod 3 = 0 [
      set annual-KC-cover-before-soy-values lput ? annual-KC-cover-before-soy-values
    ]
    set counter-cover-before-soy-values counter-cover-before-soy-values + 1
  ]
  ;;;;;;
  ;; Determine when the max KC is and make the critical growing period +- 30 days
  ;;;;;;  
  let max-KC-cover-before-soy max annual-KC-cover-before-soy-values
  let day-max-kc-cover-before-soy position max-KC-cover-before-soy annual-KC-cover-before-soy-values
  set critical-grow-cover-before-soy-start day-max-kc-cover-before-soy - 30
  set critical-grow-cover-before-soy-end day-max-kc-cover-before-soy + 31 ;; one day beyond 30, so ensure +- 30 days
  
  ;;;;;;
  ;; Determine when cover harvest happens
  ;;;;;;
  let counter-cover-KC 0
  let previous-kccover-here 0
  set day-harvest-cover-before-soy 0
  
  while [ day-harvest-cover-before-soy = 0 ] [
    let kc-value item counter-cover-KC annual-KC-cover-before-soy-values ;; record the KC value corresponding to the day from the counter
    if kc-value = 0 and previous-kccover-here > 0 [ ;; current value 0 but previous was above 0
      set day-harvest-cover-before-soy counter-cover-KC - 1 ;; the previous day was the day to harvest
    ] 
    set previous-kccover-here kc-value
    set counter-cover-KC counter-cover-KC + 1
  ]

  ;;;;;;;;;;;;;;;;;;;;;;;;  
  ;;; WHEAT BEFORE SOY ;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;
  ;;; creating a list of this year's KC wheat-before-soy values
  ;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  let annual-KC-wheat-before-soy-values []
  let counter-wheat-before-soy-values 0
  foreach wheat-before-soy-values [
    if counter-wheat-before-soy-values mod 3 = 0 [
      set annual-KC-wheat-before-soy-values lput ? annual-KC-wheat-before-soy-values
    ]
    set counter-wheat-before-soy-values counter-wheat-before-soy-values + 1
  ]
  ;;;;;;
  ;; Determine when the max KC is and make the critical growing period +- 30 days
  ;;;;;;  
  let max-KC-wheat-before-soy max annual-KC-wheat-before-soy-values
  let day-max-kc-wheat-before-soy position max-KC-wheat-before-soy annual-KC-wheat-before-soy-values
  set critical-grow-wheat-before-soy-start day-max-kc-wheat-before-soy - 30
  set critical-grow-wheat-before-soy-end day-max-kc-wheat-before-soy + 31 ;; one day beyond 30, so ensure +- 30 days
  ;;;;;;
  ;; Determine when wheat harvest happens
  ;;;;;;
  let counter-wheat-KC 0
  let previous-kcwheat-here 0
  set day-harvest-wheat-before-soy 0
  
  while [ day-harvest-wheat-before-soy = 0 ] [
    let kc-value item counter-wheat-KC annual-KC-wheat-before-soy-values ;; record the KC value corresponding to the day from the counter
    if kc-value = 0 and previous-kcwheat-here > 0 [ ;; current value 0 but previous was above 0
      set day-harvest-wheat-before-soy counter-wheat-KC - 1 ;; the previous day was the day to harvest
    ] 
    set previous-kcwheat-here kc-value
    set counter-wheat-KC counter-wheat-KC + 1
  ]
  
  ;;;;;;;;;;;;;;;;;;;;;;;
  ;;; SHORT CYCLE SOY ;;;
  ;;;;;;;;;;;;;;;;;;;;;;; 
  ;;;
  ;;; creating a list of this year's KC short-cycle-soy values
  ;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  let annual-KC-short-cycle-soy-values []
  let counter-short-cycle-soy-values 0
  foreach short-cycle-soy-values [
    if counter-short-cycle-soy-values mod 3 = 0 [
      set annual-KC-short-cycle-soy-values lput ? annual-KC-short-cycle-soy-values
    ]
    set counter-short-cycle-soy-values counter-short-cycle-soy-values + 1
  ]
  ;;;;;;
  ;; Determine when the max KC is and make the critical growing period +- 30 days
  ;;;;;;  
  let max-KC-short-cycle-soy max annual-KC-short-cycle-soy-values
  let day-max-kc-short-cycle-soy position max-KC-short-cycle-soy annual-KC-short-cycle-soy-values
  set critical-grow-short-cycle-soy-start day-max-kc-short-cycle-soy - 30
  set critical-grow-short-cycle-soy-end day-max-kc-short-cycle-soy + 31 ;; one day beyond 30, so ensure +- 30 days
  ;;;;;;
  ;; Determine when short-cycle-soy starts growing and when check WT depth to see if short-cycle-soy will be planted
  ;; To do this, mark what day the KC for short-cycle-soy first is > 0
  ;;;;;;
  let counter-short-cycle-soy-KC 0
  let previous-kcshort-cycle-soy-here 0
  let day-plant-short-cycle-soy 0
  while [ day-plant-short-cycle-soy = 0 ] [
    let kc-value item counter-short-cycle-soy-KC annual-KC-short-cycle-soy-values ;; record the KC value corresponding to the day from the counter
    if kc-value > 0 and previous-kcshort-cycle-soy-here = 0 [
      set day-plant-short-cycle-soy counter-short-cycle-soy-KC ;; this is the year growth started
    ]
    set previous-kcshort-cycle-soy-here kc-value
    set counter-short-cycle-soy-KC counter-short-cycle-soy-KC + 1
  ]    
  set day-check-for-short-cycle-soy day-plant-short-cycle-soy - 7 ;; check  7 days before the crop will start growing
end

to change-world-dimensions
  ;; Sizes the landscape to match a) input file dimensions or b) sliders regarding dimensions
  
  ifelse Import-landscape? [
    ;; Use landscape size based on import list
    ;; Assumes a square landscape
    let number-patches ( (length imported-landscape-list ) - 5 ) / 5
    let dimensions sqrt number-patches
    let minmax-dimensions int ( dimensions / 2 )
    resize-world ( - minmax-dimensions ) minmax-dimensions ( - minmax-dimensions ) minmax-dimensions
    
  ]
  [
    ;; Uses sliders to resize world
    ;; Assumes a square landscape
    resize-world ( - Landscape-size ) Landscape-size ( - Landscape-size ) Landscape-size
  ]
end

to-report find-center-patch
  ;; 1. World metrics validation
  ;; To simplify the algorithm (TODO: Ver si estas asunciones son muy restrictivas):
  ;; - The world must be square with an odd number of patches (so, that there is only one center patch and the elevation at the borders is the same).
  if (world-width != world-height) [
    print "The world must be square"
    stop
  ] 
  if (((count patches) mod 2) = 0) [
    print "The world must have and odd number of patches"
    stop
  ]

  ;; 2. Find center patch (based on assumption of item 1) an
  let center-pxcoord (max-pxcor + min-pxcor) / 2
  let center-pycoord (max-pycor + min-pycor) / 2
  let center-patches patches with [ pxcor = center-pxcoord and pycor = center-pycoord ]
  report (one-of center-patches)
end

to setup-elevation
  ;; Sets the elevation of the landscape, either a) from file or b) generic slop by slider; creates list of all patches and all patches' neighbors
  
  ifelse Import-landscape? [
    ;; Setting landscape from imported list
    ;; The list has the has x and y coordinates for each patch and the corresponding elevation
    let patch-counter 1
    repeat count patches [
      let patch-updating patch ( item ( ( patch-counter * 5 ) ) imported-landscape-list ) ( item ( ( patch-counter * 5 ) + 1 ) imported-landscape-list )
      ask patch-updating [
        set elevation precision ( item ( ( patch-counter * 5 ) + 3 ) imported-landscape-list ) 4
      ]
      set patch-counter patch-counter + 1
    ]
  ]
  [
    ;;; Sets the elevation using a generic slope from a slider with the landscape sloped from left (high) the right (low)
    let slope-percent slope / 100
    let cell-dimension sqrt surface-cell  
    ask patches [
      let pycor-here pycor
      let elevation-multiplier distance patch max-pxcor pycor-here
      ;; adjusts the elevation of all cells based upon the slope-percent, set by a slider, and the elevation-multiplier.    
      set elevation precision ( elevation-multiplier * Slope-percent * cell-dimension  ) 4     
    ]
  ]
  
  ;; Now creating lists of patches sorted by elevation (small to large) for the overall landscape and of each patch's neighbors.
  set Landscape-Elevation-list sort-on [ elevation ] patches
  ask patches [
    set Neighbor-Elevation-list sort-on [ elevation ] neighbors
  ]
end

to setup-soil
  ;; Sets specific yield and the depth of the soil column
 
  ask patches [
    ;; Medium pore specific yield set by slider
    set large-pore-specific-yield precision ( groundwater-specific-yield - medium-pore-specific-yield ) 4 ;; Groundwater = medium + large
    set soil-bottom -100 ;; The bottom of the soil column is 100 m below sea level
  ]
end

to setup-initial-groundwater
  ;; Setup of bounrary conditions. initializes the groundwater levels, any initial surface water resulting from the initial groundwater water level, and the capillary zone, which is directly dependent on the groundwater levels
  
  ask patches [
    
    setup-boundary ;; Setup special boundary conditions if this watershed is not self contained i.e. the groundwater levels rise and fall without regard to what happens in the area outside this landscape 
    
    ;; Groundwater zone
    setup-initial-head ;; setup initial groundwater level and, if the initial groundwater level is above the elevation, then initial surface water.
    
    ;;;; Capillary Zone - setup dependent on the size of the groundwater zone
    set capillary-top precision ( groundwater-top + 0.8 ) 4 ;; The first 0.8 meters of soil will be the capillary zone 
    if capillary-top > elevation [ set capillary-top elevation ] ;; If the capillary top would be higher than the elevation, reset it to be the elevation
  ]
end

to setup-boundary
  ;; Setup special boundary conditions if this watershed is not self contained i.e. the groundwater levels rise and fall without regard to what happens in the area outside this landscape 
  
  ifelse groundwater-self-contained? = true [
    ;; No cells will have special boundary conditions   
    set boundary-cell? false    
  ]
  [
    ;; The groundwater in the cells on the outside of the landscape are connected to the water table that surrounds the landscape and thus will maintain a constant head
    ifelse count neighbors < 8 [ ;; Less than 8 neighbors so it is an edge cell
      set boundary-cell? true 
    ]
    [
      set boundary-cell? false ;; Not an edge cell
    ]
  ]
end

to setup-initial-head
  ;; setup initial groundwater level and, if the initial groundwater level is above the elevation, then initial surface water.

    ;;; Check if groundwater level is above elevation, and assign excess to surface water
    ifelse Initial-absolute-water-level > elevation [
      ;;; If initial water level is above the soil
      set groundwater-top precision elevation 4
      set surface-water-column precision ( Initial-absolute-water-level - elevation ) 4 ;; water is on the surface
    ]
    [
      ;;; If initial water level below or at the surface
      set groundwater-top Initial-absolute-water-level
      set surface-water-column 0
    ]
    
    ;; Now calculate the groundwater water column
    set groundwater-water-column precision ( ( groundwater-top - soil-bottom ) * groundwater-specific-yield ) 4 ;; height without soil - medium and large poresss
    ;show water-table-height checked 7/5/13

  ;; Record initial values for boundary cells
  if boundary-cell? = true [
    set initial-groundwater-top groundwater-top
    set initial-surface-water-column surface-water-column
    set initial-groundwater-water-column groundwater-water-column
  ]

  
end

to setup-upper-and-root-zones
  ;; This sets the initial soil and water column levels in the upper and root zones
  ask patches [
      
   ;; Determine the root zone bottom
   ifelse ( precision ( elevation - capillary-top ) 4 ) > .3 and ( precision ( elevation - capillary-top ) 4) > root-length [ ;; As long as the soil column is at least 0.3 and the root does not go into the CZ...
                                                                            ;; The  bottom will be the elevation minus whichever is larger, the root length or 0.3 m
                                                                            ;; We are subtracting what will be the soil column from the elevation to get the bottom
                                                                            ;; Otherwise, the bottom is the the elevation minus the soil height above the capillary zone
                                                                            ;; We condensed this into an "and" rather than having the soil-column-above-CZ > root-length be
                                                                            ;; a sub condition because the "else" in both cases is set root-zone-soil-column soil-column-above-CZ
    set root-zone-bottom precision ( elevation - ( max (list .3 root-length ) ) ) 4 ;; 
  ]
  [ ;; the else
    ;; (a) soil-column-above-CZ > .3  but soil-column-above-CZ <= root length or (b) soil-column-above-CZ <= .3 (and any root length)
    ;; the soil-column-above-CZ is smaller than the RZ minimum of .3 or the root descends into the CZ
    set root-zone-bottom capillary-top ;; the root bottom will be the same as the capilalry top
  ]  

   ;; The water columns for the root and upper zones based on medium pore specific yield and initial medium pore saturation
    set root-zone-water-column precision ( ( elevation - root-zone-bottom ) * medium-pore-specific-yield * Initial-saturation-medium-pores ) 4
    set upper-zone-water-column precision ( ( root-zone-bottom - capillary-top ) * medium-pore-specific-yield * Initial-saturation-medium-pores ) 4
    
    set RZ-UZ-over 0 ;; This variable is used in water accounting for error checking to endure the root and upper zone saturations do not go above 100%    
  ]
end

to setup-farms-crops
  ;; Sets up patch variables used during the run relating to the crop cycling and yields
  ask patches [
    set dead? true
    set stressed? false
    set last-day-0-decision ""
    set Rotation-year 1 ;; 0 = 
    plant-fallow
    set planting-history ( list Planting-scenario ( word "year" year ) ) ;; Sets sup the planting history variable
    set day-harvest ""
  ]
end

to setup-crop-yield
  ;; Sets values related to critical growing periods and yield for each crop
        
  set critical-growing-period-pasture? true ;; This can be set here since pasture is always in the critical growing period. The over covers are variable depending on the time of year
  set critical-growing-period-cover-before-soy? false ;; All other crops set to false and then changed during the year
  set critical-growing-period-soy? false 
  set critical-growing-period-wheat-before-soy? false
  set critical-growing-period-short-cycle-soy? false  
  set critical-growing-period-maize? false
  
  ;; Setup yields
  ;; Tonnage per hectare
  set soy-tonnage-per-hectare 5.5
  set pasture-tonnage-per-hectare 8
  set maize-tonnage-per-hectare 14
  set cover-before-soy-tonnage-per-hectare 5
  set wheat-before-soy-tonnage-per-hectare 6
  set short-cycle-soy-tonnage-per-hectare 3.5
  ;; Convert to tonnage per patch
  let patch-percent-hectare precision ( surface-cell / 10000 ) 4 ;; Portion of patch area to hectare
  set soy-tonnage-per-patch precision ( soy-tonnage-per-hectare * patch-percent-hectare ) 4
  set pasture-tonnage-per-patch precision ( pasture-tonnage-per-hectare * patch-percent-hectare ) 4
  set maize-tonnage-per-patch precision ( maize-tonnage-per-hectare * patch-percent-hectare ) 4
  set cover-before-soy-tonnage-per-patch precision  ( cover-before-soy-tonnage-per-hectare * patch-percent-hectare ) 4
  set wheat-before-soy-tonnage-per-patch precision  ( wheat-before-soy-tonnage-per-hectare * patch-percent-hectare ) 4
  set short-cycle-soy-tonnage-per-patch precision ( short-cycle-soy-tonnage-per-hectare * patch-percent-hectare ) 4
end

to apply-patch-labels
  ;; This has the patches label themselves with a variable's value indicated by a chooser

  ask patches [    
    if labeled-variable = "land-cover" [
      set plabel land-cover
    ]
    if labeled-variable = "dead?" [
      set plabel dead?
    ] 
    if labeled-variable = "total-water-volume" [
      set plabel precision ( patch-total-water-height ) 0
    ]
    if labeled-variable = "groundwater-top + surface water" [
      set plabel precision ( groundwater-top + surface-water-column ) 3
    ]
    if labeled-variable = "surface-water-column" [
      set plabel precision surface-water-column 4
    ]
    if labeled-variable = "root-zone-bottom" [
      set plabel precision root-zone-bottom 3
    ]
    if labeled-variable = "root-zone-soil-column" [
      set plabel precision ( elevation - root-zone-bottom ) 3
    ]
    if labeled-variable = "root-zone-water-column" [
      set plabel precision root-zone-water-column 3
    ]
    if labeled-variable = "upper-zone-water-column" [
      set plabel precision upper-zone-water-column 3
    ]
    if labeled-variable = "root-zone-water-column+upper-zone-water-column" [
      set plabel precision ( upper-zone-water-column + root-zone-water-column ) 3
    ]
    if labeled-variable = "upper-zone-soil-column" [
      set plabel precision ( root-zone-bottom - capillary-top ) 3
    ]
    if labeled-variable = "capillary-top" [
      set plabel precision capillary-top 3
    ]
    
    if labeled-variable = "capillary-soil-column" [
      set plabel precision ( capillary-top - groundwater-top ) 3
    ]
    if labeled-variable = "soil-column-above-groundwater" [
      set plabel precision ( elevation - groundwater-top ) 3
    ]
    if labeled-variable = "groundwater-top" [
      set plabel precision groundwater-top 3
    ]
    if labeled-variable = "groundwater-water-column" [
      set plabel precision groundwater-water-column 3
    ]   
    if labeled-variable = "root-bottom" [
      set plabel precision root-bottom 3
    ]
    if labeled-variable = "root-length" [
      set plabel precision root-length 3
    ]  
    if labeled-variable = "percent-roots-in-root-zone" [
      set plabel precision ( percent-roots-in-root-zone * 100 ) 0
        ]
    if labeled-variable = "percent-roots-in-capillary-zone" [
      set plabel precision ( percent-roots-in-capillary-zone  * 100 ) 0
    ]
    
    if labeled-variable = "ETc" [
      set plabel precision ETc 4
    ]
    if labeled-variable = "daily-unmet-need" [
      set plabel precision daily-unmet-need 4
    ]
    
    if labeled-variable = "annual-yield" [
      ifelse is-number? annual-yield [
        set plabel precision annual-yield 4
      ]
      [
        set plabel 0
      ]
    ]
    if labeled-variable = "cumulative-percent-daily-unmet-need" [
      set plabel precision cumulative-percent-daily-unmet-need 4
    ]  
    if labeled-variable = "water-available-root-zone" [
      set plabel precision water-available-root-zone 4
    ]
    if labeled-variable = "water-available-capillary-zone" [
      set plabel precision water-available-capillary-zone 4
    ]
    if labeled-variable = "transpiration-amount" [
      set plabel precision transpiration-amount 4
    ]
    if labeled-variable = "evaporation-amount" [
      set plabel precision evaporation-amount 4
    ]
    if labeled-variable = "top of water" [

      ifelse surface-water-column > 0 [
        set plabel precision ( surface-water-column + elevation ) 3
      ]
      
      [
        set plabel precision groundwater-top 2
      ]
      
    ]
    if labeled-variable = "elevation" [
      set plabel precision elevation 3
    ]
    if labeled-variable = "soil-depth" [
      set plabel precision soil-bottom 0
    ]
    if labeled-variable = "crop yield factor" [
     set plabel precision ( crop-yield-factor * 100 ) 0
    ]
    
    if labeled-variable = "days-no-water" [
      set plabel days-no-water
    ]
    if labeled-variable = "farmId" [
      set plabel farmId
    ]
    if labeled-variable = "days-current-crop-critical" [
      set plabel days-current-crop-critical
    ]
  ]
end

to apply-colors
  ask patches [
    ;; This has the patches color themselves based on land cover and the crop's status and produces an icon if there is surface water
    
    if land-cover = "fallow"
    [
      set pcolor grey
    ]
    if land-cover = "pasture"
    [
      set pcolor red
    ]
    if land-cover = "soy"
    [
      set pcolor orange
    ]
    
    if land-cover = "maize"
    [
      set pcolor brown
    ]
    
    if land-cover = "cover-before-soy"
    [
      set pcolor yellow
    ]
       
    if land-cover = "wheat-before-soy"
    [
      set pcolor sky
    ]
    
    if land-cover = "short-cycle-soy"
    [
      set pcolor violet
    ]
    ifelse dead? [
      set pcolor pcolor - 4
    ]
    [
      if stressed? [
        set pcolor pcolor - 2 
      ]
    ]
    
    ifelse surface-water-column > 0 [
      ifelse any? turtles-here [
      ]
      [
       sprout 1 [ set shape "puddle" ] 
      ]
      
    ]
    [
     if any? turtles-here [ ask turtles-here [ die ] ] 
    ]
  ]
end

to setup-export
  ;; Setup the export file if the option to export patch level data to a separate file during a run is activated
  
  ;; Open a timestamped file and create a the headings for all the variables that will be exported later
  file-open (word (word "PampasRun_iWTD=" Initial-absolute-water-level "_WS=" Water-Scenario "_PlantScen=" Planting-scenario "_" (remove "-" (remove " " (remove "." (remove ":" date-and-time)))) ".txt"))
      ;; file-open (word (word "run" behaviorspace-run-number (remove "-" (remove " " (remove "." (remove ":" date-and-time)))) ".txt"))  
                                                                                                               ;let variable-list (word pycor "," ticks "," groundwater-top ","))
                                                                                                               ;file-print (word pxcor "," pycor "," ticks "," groundwater-top ",")
  file-write "WTD" ;elevation - groundwater-top
  file-write "surface-water-column"
;  file-write "pxcor"
;  file-write "pycor"
;  file-write "ticks"
;  file-write "year"
;  file-write "day"
;  file-write "last-day-0-decision"
  file-write "land-cover"
;  file-write "surface-water-column"
;  if export-root-zone-soil-column? [ file-write "root-zone-soil-column" ]
;  if export-root-zone-water-column? [ file-write "root-zone-water-column" ]
;  if export-upper-zone-water-column? [ file-write "upper-zone-water-column" ]
;  if export-upper-zone-soil-column? [ file-write "upper-zone-soil-column" ]
;  file-write "soil-column-above-groundwater"
;  if export-groundwater-top? [ file-write "groundwater-top" ]
;  if export-groundwater-water-column?  [ file-write "groundwater-water-column" ]
;  if export-roots-damaged? [ file-write "roots-damaged?" ]
;  if export-root-bottom? [ file-write "root-bottom" ]
;  file-write "root-length"
;  file-write "percent-roots-in-root-zone"
;  file-write "ETc"
;  file-write "water-available-root-zone"
;  file-write "water-available-capillary-zone"
;  file-write "transpiration-amount"
;  file-write "evaporation-amount"
;  file-write "ETc"
;  file-write "water-deficit"
;  file-write "cumulative-percent-daily-unmet-need"
;  file-write "elevation"
;  ;     if export-crop-yield-factor? [ file-write crop-yield-factor ]
;  if export-dead? [ file-write "dead?" ]
;  if export-tonnage-per-hectare?  [
;    file-write "soy-tonnage-per-hectare"
;    file-write "pasture-tonnage-per-hectare"
;    file-write "cover-before-soy-tonnage-per-hectare"
;    file-write "wheat-before-soy-tonnage-per-hectare"
;    file-write "short-cycle-soy-tonnage-per-hectare"
;    file-write "maize-tonnage-per-hectare"
;  ]
;  file-write "daily-unmet-need"
;  file-write "cumulative-percent-daily-unmet-need"
  file-write "annual-yield"
;  file-write "years-soy-planted"
;  file-write "years-pasture-planted"
;  file-write "years-cover-before-soy-planted"
;  file-write "years-wheat-before-soy-planted"
;  file-write "years-short-cycle-soy-planted"
;  file-write "years-maize-planted"
;  file-write "patch-soy-output"
;  file-write "patch-pasture-output"
;  file-write "patch-cover-before-soy-output"
;  file-write "patch-wheat-before-soy-output"
;  file-write "patch-short-cycle-soy-output"
;  file-write "patch-maize-output"
;  
;  if export-boundary-cell?  [ file-write "boundary-cell?"]
;  file-write "daily-rain-amount"
;  file-write "daily-ETo"
;  if export-daily-Kcb-values? [
;    file-write "daily-Kcb-soy"
;    file-write "daily-kcb-pasture"
;    file-write "daily-kcb-cover-before-soy"
;    file-write "daily-Kcb-wheat-before-soy"
;    file-write "daily-Kcb-short-cycle-soy"
;    file-write "daily-Kcb-maize"
;  ]
;  if export-daily-root-lengths? [
;    file-write "daily-soy-root-length"
;    file-write "daily-pasture-root-length"
;    file-write "daily-cover-before-soy-root-length"
;    file-write "daily-wheat-before-soy-root-length"
;    file-write "daily-short-cycle-soy-root-length"
;    file-write "daily-maize-root-length"
;  ]
;  if export-total-water-volume?  [ file-write "total-water-volume"]
;  file-write "farmId"
  file-print ""    
end

           ;;
         ;;;;;; 
       ;;;;;;;;;;; 
     ;;;;;;;;;;;;;;
   ;;;;          ;;;;
 ;;;;;;    GO    ;;;;;;
   ;;;;          ;;;;
     ;;;;;;;;;;;;;;
       ;;;;;;;;;;; 
         ;;;;;; 
           ;;

to go
  
  If print-procedure-in-code? [ 
    print ""
    print ""
    type "ticks = " print ticks
    print "start of new day"
  ]


  update-daily-values   ;; This section sets the daily values for rain and ETo and, for each crop type, the daily KCB, root length, and potential root regrowth
  
  ;; Water processes
  
  rain-infiltration ;; Determines how much of precipitation will attempt to attempt to infiltrate immediately and infiltrate the new water and any previous water n the surface. Adds runoff to the surface at the end of the procedure.

  surface-flow ;; Water flow is based on equilibrium levels and the flows repeat to ensure equilibrium across the entire landscape is reached. Flow happen asynchronously.

  if no-groundwater-flow? = false [ ;; If the switch disabling groundwater slow is turned off, then have the water flow
    groundwater-flow ;; Asynchronous updating of groundwater head (groundwater top) followed by an update of teh capillary zone
  ]
  
  ;; Plant processes
  
  ;; If it's a planting decision day...
  ;; There is always a decision on day 0 regardless of scenario, but there are additional decisions made about soy plantings later in the year in the individual adaptation scenario
  if day = 0 [ planting-decision-day0 ] ;; Makes the day 0 decision about what to plant
  
  ;; Checking to see if it is the individual adaptation scenario
  ifelse planting-scenario = "Individual Adaptation" [
    ;; Checking to see if it is the particular day for a decision and then ask the patches that will recheck the water table on that day only
    if day = day-check-for-maize [
      ask patches with [ day-check-WTD = day-check-for-maize ] [
        planting-decision-individual-adaptation-maize   ;; Check for maize conditions
      ]
    ]
    if day = day-check-for-soy [
      ask patches with [ day-check-WTD = day-check-for-soy ] [
        planting-decision-individual-adaptation-full-cycle-soy   ;; Check for full cycle soy conditions
      ]
    ]
    if day = day-check-for-short-cycle-soy [
      ask patches with [ day-check-WTD = day-check-for-short-cycle-soy ] [
        planting-decision-individual-adaptation-short-cycle-soy ;; Check for short cycle soy conditions
      ]
    ]
  ]
  [
    plant-crops   ;; Plants crops based on the day 0 decision
  ]

  if no-plant-growth? = false [
    grow-plants ;;; Updating root lengths and root bottom, and of the roots changed, update the root and upper zone soil columns
  ]
  
  if no-evapotranspiration? = false [ ;; If the switch disabling evapotranspiration is turned off, then evapotranspire
    evapotranspiration ;; a) Determine plant need at the patch level, b) where the roots are, c) how much water is available for ET, d) do the transpiration, e) evaporate water, f) update the water horizons
  ]

  Harvest ;; Updated land cover after previous day's harvest, calculates the crop yield factor and yield for patches when their harvest day is the current day
  
  ;;;;;;;;;;;;;;;;;;;
  ;; Data tracking ;;
  ;;;;;;;;;;;;;;;;;;;
  
  ;;; update labels, patch colors, and plots
  calculate-globals   ;; Global variables used in plotting and error tracking
  apply-patch-labels  ;; This has the patches label themselves with a variable's value indicated by a chooser
  apply-colors        ;; This has the patches color themselves based on land cover and the crop's status and produces an icon if there is surface water
  do-plots            ;; Plot data in graphs in the interface
  
  if year >= Export-start-year and Export-data? = true [
    export-data   ;; If the option to export patch level data to a separate file during a run is activated, then this exports the data to the file opened during setup
  ]
  
  ;;;;;;;;;;
  ;; Time ;;
  ;;;;;;;;;;
  
  tick ;; advance the tick (iteration) counter
  set day day + 1 ;; Advance the counter for the days in the current year
  
  ;; If it is the end of the year, reset the year
  if Day > days-in-year [
    reset-year ;; Reset time variables, advance the year counter, and updates the shortened annual list from the larger input files
  ]
  
  ;;;;;;;;;;;;;;;;;;;;;
  ;; Stop conditions ;;
  ;;;;;;;;;;;;;;;;;;;;;

  if year >= Years-of-run
    [
      calculate-output-totals ;; Sum the outputs of each crop for the entire landscape and determine the maximum potential output
      if Export-data? = true [ ;; Close the export file
        file-close
      ]
      stop
    ]
end

to update-daily-values
  ;; This section sets the daily values for rain and ETo and, for each crop type, the daily KCB, root length, and potential root regrowth
  If print-procedure-in-code? [ print "update-daily-values" ]
  
  set error-code "update-daily-values" ;; Debugging code
  
  ;; Setting the daily rain amount from three different possibilities
  ifelse no-rain? = false [
    ifelse ONEcm-rain? = false [
      ;; Rainfall amount set by the input file of the selected scenario
      set daily-rain-amount precision ( item ( day * 3 ) current-year-values ) 4 ;; selecting the proper item in the list based on the day and column of rainfall
    ]
    [
      ;; Fixed daily rainfall of 1 cm
      set daily-rain-amount 0.01
    ]
  ]
  [
    ;; This option makes no rain
    set daily-rain-amount 0
  ]
  
  ;; Setting the ETo value from the input file based on the current day and column that contains the ETo data
  set daily-ETo ( item ( ( day * 3 ) + 1 ) current-year-values )
  
  ;;; Setting the daily KCB values for each crop  
  ( foreach crop-list [
    ;; generates a sentence like: "set daily-Kcb-______ ( item ( day * 3 ) _______-values )"
    let sentence-here ( word "set daily-Kcb-" ?1 " precision ( item ( day * 3 ) " ?1 "-values ) 4" )
    run sentence-here
  ] )
  
  ;;; Setting the daily root lengths for each crop
    ( foreach crop-list [
    ;; generates a sentence like: "set daily-_______-root-length ( item ( ( day * 3 ) + 1 ) ________-values )"
    let sentence-here ( word "set daily-" ?1 "-root-length precision ( item ( ( day * 3 ) + 1 ) " ?1 "-values ) 4" )
    run sentence-here
  ] )
  
  ;;; Setting the daily root regrowth lengths for each crop
    ( foreach crop-list [
    ;; generates a sentence like:  "set daily-potential-root-regrowth-________ ( item ( ( day * 3 ) + 2 ) _________-values )"
    let sentence-here ( word "set daily-potential-root-regrowth-" ?1 " precision ( item ( ( day * 3 ) + 2 ) " ?1 "-values ) 4" )
    run sentence-here
  ] )   
end

to planting-decision-day0
  ;; Makes the day 0 decision about what to plant
  If print-procedure-in-code? [ print "planting-decision-day0" ]
    
  ;; If a) we are using an imported landscape and it is year 0 or b) it is planing scenario 0 or c) it is an individual adaptation scenario then...
  ifelse ( Import-landscape? = true and year = 0 ) or ( Planting-scenario = "0" ) or ( planting-scenario = "Individual Adaptation" )[
    ;; If we are using imported cover and it is year 0, setup the landscape from the file
    ifelse ( Import-landscape? = true and year = 0 ) [
      setup-imported-cover ;; Reads the cover from the input file and sets the last-day-0-decision variable
    ] 
    [
      ifelse planting-scenario = "Individual Adaptation" [
        planting-decision-individual-adaptation-day0 ;; Looks at the water table and determines if a crop is planted now and when to recheck the water table
      ]
      [
        setup-cover-from-sliders ;; Sets up the cover randomly from the sliders
      ]
    ]
  ] 
  [
    ;; All other cases, where the cover remains the same or follows a prescribed strict rotation or cycle
    ask patches [
      if Planting-scenario = "Strict Rotation" [
        planting-decision-strict-rotation-day0 ;; Fixed rotation of three covers: 1) maize, 2) soy, 3) wheat then soy
      ]
      
      if Planting-scenario = "Observed Sequence" [
        planting-decision-observed-sequence-day0 ;; A five year sequence: 1) wheat then soy, 2) maize, 3) soy, 4) wheat then soy, 5) maize
      ]
      
      ;; Scenarios with all the same cover everywhere the entire simulation
      
      if Planting-scenario = "1a" and day = 0 [ ;; all soy
        set last-day-0-decision "soy"
      ]
      
      if Planting-scenario = "1b" and day = 0 [ ;; all pasture
        set last-day-0-decision "pasture"
        plant-pasture
      ]
      
      if Planting-scenario = "1c" and day = 0 [ ;; all pasture
        set last-day-0-decision "maize"
      ]
      
      if Planting-scenario = "1d" [ ;; all cover then soy
        set last-day-0-decision "cover-soy"
        plant-cover-before-soy
      ]
      
      if Planting-scenario = "1e" [ ;; all wheat then soy
        set last-day-0-decision "wheat-soy"
        plant-wheat-before-soy
      ]
      
      if Planting-scenario = "1f" and day = 0 [ ;; fallow
        set last-day-0-decision ""
        plant-fallow
      ]
    ]
  ]
end

to setup-imported-cover
  ;; Reads the cover from the input file and sets the last-day-0-decision variable
  
  if day = 0 [
    If print-procedure-in-code? [ print "setup-imported-cover" ]
    let patch-counter 1
    repeat count patches [
      let patch-updating patch ( item ( ( patch-counter * 5 ) ) imported-landscape-list ) ( item ( ( patch-counter * 5 ) + 1 ) imported-landscape-list )
      ask patch-updating [
        let initialLandUse ( item ( ( patch-counter * 5 ) + 4 ) imported-landscape-list )
        if initialLandUse = 1 [
          set last-day-0-decision "pasture"
          plant-pasture
        ]
        if initialLandUse = 2 [
          set last-day-0-decision "soy"
        ]
        if initialLandUse = 3 [
          set last-day-0-decision "cover-soy"
          plant-cover-before-soy
        ]
        if initialLandUse = 4 [
          set last-day-0-decision "" ;; fallow
        ]
        if initialLandUse = 5 [
          set last-day-0-decision "maize"
        ]
        if initialLandUse = 6 [
          set last-day-0-decision "wheat-soy"
          plant-wheat-before-soy
        ]
        set farmId ( item ( ( patch-counter * 5 ) + 2 ) imported-landscape-list )
      ]
      set patch-counter patch-counter + 1
    ]
  ] 
end

to planting-decision-individual-adaptation-day0
  ;; Looks at the water table and determines if a crop is planted now and when to recheck the water table
  If print-procedure-in-code? [ print "planting-decision-individual-adaptation-day0" ]
  ask patches [
    let soil-column-above-groundwater precision ( elevation - groundwater-top ) 4
;
    ifelse ( ( ( precision soil-column-above-groundwater 1 ) > May-Dec-ST ) and ( precision soil-column-above-groundwater 1 ) < May-DT )
      or ( Rotation-year = 3 and ( precision soil-column-above-groundwater 1 ) > May-Dec-ST )
    ;; WTD > shallow threshold but WTD < the deeper threshold, ie in the danger zone
    ;; OR it's year three and only WTD > the shallow threshold
    ;; Then there will be a crop planted on day 0
    [
      ;;; There will be two plantings. Determine if it will be wheat or cover first.
      ifelse last-day-0-decision = "wheat" [ ;; Do not plant wheat in immediately consecutive years
;        print "plant cover"
        ;; plant cover
        plant-cover-before-soy ;; plant the crop
        set last-day-0-decision "cover" ;; record the decision
        set day-check-WTD day-check-for-soy ;; set when to check the WTD next
        if pxcor = 0 and pycor = 0 [
          output-type land-cover output-type " "
        ]
      ]
      [
;        print "plant wheat"
        ;; plant wheat
        plant-wheat-before-soy
        set last-day-0-decision "wheat"
        set day-check-WTD day-check-for-short-cycle-soy
        if pxcor = 0 and pycor = 0 [
          output-type land-cover output-type " "
        ]
      ] 
    ]
    [
;      print "do nothing now"
      ;; No planting now. Set the date when the patch will next check the WTD
      set last-day-0-decision "" ;; reset the day 0 variable, which indicates no day 0 planting occurred
      ifelse Rotation-year = 1 [
        ;; If it is rotation year 1, check for maize next
        set day-check-WTD day-check-for-maize
      ]
      [
        ;; Otherwise, check for regular soy next 
        set day-check-WTD day-check-for-soy
      ]
    ]
  ]
end

to setup-cover-from-sliders
  ;; Sets up the cover randomly from the sliders
  
  if year = 0 [
    If print-procedure-in-code? [ print "setup-cover-from-sliders" ]
    ;; Only makes the decision in year 0, in later years it just repeats
    ask patches [    
      ;; Draw a random number and set cover accordingly as either soy or pasture
      set last-day-0-decision ""
      let chance-soy random-float 1
      ifelse chance-soy <(  %-Soy / 100 ) [
        set last-day-0-decision "soy"
      ]
      [ ;;; pasture        
        set last-day-0-decision "pasture"
      ]
    ]
    
    let num-cover-patches ( %-Cover-Crop / 100 ) * count patches
    ;; Draw a random number and convert pasture to to cover-soy
    ;; this eliminates an error that stops the program if there are not enough patches to turn from pasture to cover
    if count ( patches with [ last-day-0-decision = "pasture" ] ) < num-cover-patches [ set num-cover-patches ( count patches with [ last-day-0-decision = "pasture" ] ) ]
    ask n-of num-cover-patches patches with [ last-day-0-decision = "pasture" ] [
      set last-day-0-decision "cover-soy"
    ]
  ]
  
  ;; Now plant the crops
  ask patches with [ last-day-0-decision = "pasture" ] [
    plant-pasture
  ]
  ask patches with [ last-day-0-decision = "wheat-soy" ] [
    plant-wheat-before-soy
  ]
  ask patches with [ last-day-0-decision = "cover-soy" ] [
    plant-cover-before-soy
  ]
end


to planting-decision-strict-rotation-day0
  ;; Fixed rotation of three covers: 1) maize, 2) soy, 3) wheat then soy
  If print-procedure-in-code? [ print "planting-decision-strict-rotation-day0" ]
  ifelse rotation-year = 1 [
    ;; Maize
    set last-day-0-decision "maize"
    set rotation-year rotation-year + 1 ;; advance the rotation
  ]
  [
    ifelse rotation-year = 2 [
      ;; Soy
      set last-day-0-decision "soy"
      set rotation-year rotation-year + 1 ;; advance the rotation
    ]
    [
      ;; year = 3 
      ;; Wheat now, soy later
      set last-day-0-decision "wheat-soy"
      set rotation-year rotation-year + 1 ;; advance the rotation (reset elsewhere)
      plant-wheat-before-soy
    ]
  ]
end

to planting-decision-observed-sequence-day0
  ;; A five year sequence: 1) wheat then soy, 2) maize, 3) soy, 4) wheat then soy, 5) maize
  If print-procedure-in-code? [ print "planting-decision-observed-sequence-day0" ]
  if year = 0 [
    set last-day-0-decision "wheat-soy"
    plant-wheat-before-soy
  ]
  
  if year = 1 [
    set last-day-0-decision "maize"
  ]
  
  if year = 2 [
    set last-day-0-decision "soy"
  ]
  
  if year = 3 [
    set last-day-0-decision "wheat-soy"
    plant-wheat-before-soy
  ]
  
  if year = 4 [
    set last-day-0-decision "maize"
  ]
end

to planting-decision-individual-adaptation-maize
  ;; Check for maize conditions
  
  If print-procedure-in-code? [ print "planting-decision-individual-adaptation-maize" ]
  
  ;; Only plant maize if WTD is deep enough. Maize is only possible in year 1 of the rotation
  ifelse ( precision ( elevation - groundwater-top ) 1 ) > Sep-T [
    ;; plant maize
    set day-check-WTD 0
    plant-maize
    if pxcor = 0 and pycor = 0 [
      output-print land-cover
    ]
    set Rotation-year Rotation-year + 1 ;; advance the year to year 2 since planting was in accord with the rotation
  ]
  [
    ;; Check again later for regular soy
    set day-check-WTD day-check-for-soy
  ]
end


to planting-decision-individual-adaptation-full-cycle-soy
  ;; Check for full cycle soy conditions
  
  If print-procedure-in-code? [ print "planting-decision-individual-adaptation-full-cycle-soy" ]
  
  ;; Only plant soy if WTD is deep enough
  ifelse ( precision ( elevation - groundwater-top ) 1 ) > Oct-T ;; WTD deeper than the lower threshold
    [
    ;; plant soy
    set day-check-WTD 0
    plant-soy
    if pxcor = 0 and pycor = 0 [
      output-print land-cover
    ]
    ;; If the planting was in accordance with rotation, then advance the year (rotation year is (a) 2 or (b) 3 and cover planted on day 0
    if Rotation-year = 2 or ( Rotation-year = 3 and last-day-0-decision = "cover" ) [ set Rotation-year Rotation-year + 1 ]
  ]
  [
    ;; Check again later for regular soy
    set day-check-WTD day-check-for-short-cycle-soy
  ]
end

to planting-decision-individual-adaptation-short-cycle-soy
  ;; Check for short cycle soy conditions
  
  If print-procedure-in-code? [ print "planting-decision-individual-adaptation-short-cycle-soy" ]
  
  set day-check-WTD 0  ;; reset the day to check the WTD - this is the past possible planting in the year so there are no more checks this year
  ;; Only plant short cycle soy if WTD is deep enough
  ifelse ( precision ( elevation - groundwater-top ) 1 ) > May-Dec-ST ;; WTD deeper than the lower threshold
  [
    ;; plant short cycle soy
    plant-short-cycle-soy
    if pxcor = 0 and pycor = 0 [
      output-print land-cover
    ]
    if Rotation-year = 3 and last-day-0-decision != ""[ set Rotation-year Rotation-year + 1 ] ;; if year 3, as long as anything was planted on day 0, advance the year since planting was in accord with the rotation
  ]
  [
    ;; Do nothing - leave fallow
    if pxcor = 0 and pycor = 0 [
      if last-day-0-decision = "" [ output-print "fallow - no planting" ]
    ]
  ]
end

to plant-crops
  ;; Plants crops based on the day 0 decision
  ;; Does not apply with individual adaptation
  If print-procedure-in-code? [ print "plant-crops" ]
  if day = day-check-for-maize [
    ask patches with [ last-day-0-decision = "maize" ] [
      plant-maize
    ]
  ]
  
  if day = day-check-for-soy [
    If print-procedure-in-code? [ print "plant-crops" ]
    ask patches with [ last-day-0-decision = "soy" or last-day-0-decision = "cover-soy" ] [
      plant-soy
    ]
  ]
  
  if day = day-check-for-short-cycle-soy [
    If print-procedure-in-code? [ print "plant-crops" ]
    ask patches with [ last-day-0-decision = "wheat-soy" ] [
      plant-short-cycle-soy
    ]
  ]
end

to plant-soy
  If print-procedure-in-code? [ print "plant-soy" ]
  set land-cover "soy"
  set root-length 0
  set roots-damaged? False
  set years-soy-planted years-soy-planted + 1
  set day-harvest day-harvest-soy
  setup-plant-variables
end

to plant-maize
  If print-procedure-in-code? [ print "plant-maize" ]
  set land-cover "maize"
  set root-length 0
  set roots-damaged? False
  set years-maize-planted years-maize-planted + 1
  set day-harvest day-harvest-maize
  setup-plant-variables
end

to plant-cover-before-soy
  set land-cover "cover-before-soy"
  set root-length 0
  set roots-damaged? False
  set years-cover-before-soy-planted years-cover-before-soy-planted + 1
  set day-harvest day-harvest-cover-before-soy
  setup-plant-variables
end

to plant-wheat-before-soy
  If print-procedure-in-code? [ print "plant-wheat-before-soy" ]
  set land-cover "wheat-before-soy"
  set root-length 0
  set roots-damaged? False
  set years-wheat-before-soy-planted years-wheat-before-soy-planted + 1
  set day-harvest day-harvest-wheat-before-soy
  setup-plant-variables
end

to plant-short-cycle-soy
  If print-procedure-in-code? [ print "plant-short-cycle-soy" ]
  set land-cover "short-cycle-soy"
  set root-length 0
  set roots-damaged? False
  set years-short-cycle-soy-planted years-short-cycle-soy-planted + 1
  set day-harvest days-in-year
  setup-plant-variables
end

to plant-pasture
  If print-procedure-in-code? [ print "plant-pasture" ]
  set land-cover "pasture"
  if year = 0 [
    set root-length 0
    set roots-damaged? false
  ]
  set years-pasture-planted years-pasture-planted + 1
  set day-harvest days-in-year
  setup-plant-variables
end

to plant-fallow
  If print-procedure-in-code? [ print "plant-fallow" ]
  set land-cover "fallow"
  set root-length 0
  set roots-damaged? true      
  ;; set planting-history lput land-cover planting-history ;; We do not need to record fallow cover in the history
end

to setup-plant-variables
  If print-procedure-in-code? [ print "setup-plant-variables" ]
  ;; Sets up variables used by all crops and adds current planting to a planting history
  set dead? false
  set water-deficit 0
  set days-no-water 0
  set stressed? false
  set days-current-crop-critical 0
  set cumulative-percent-daily-unmet-need 0
  set annual-yield " "
  ;;; Assuming that even if there is water within 0.05 m or on the surface at the start, the plants are healthy. Processes during the run will determine if they stay alive.
  set root-bottom precision ( elevation - root-length ) 4 ;;; setting the depth of the roots in absolute terms  
  set planting-history lput ( word "day" day ) planting-history
  set planting-history lput ( elevation - groundwater-top ) planting-history
  set planting-history lput land-cover planting-history
end

to grow-plants
  ;;; Updating root lengths and root bottom and then if the roots changed, update the root and upper zones
  If print-procedure-in-code? [ print "grow-plants" ]
  
  ;; Debugging code
  ;; Sum up the water in the landscape
  set error-code "plant growth"
  
  ask patches with [ dead? = false ] [  ;;;; only update patches that are living
                                        ;;; updating the root length first based on growing season
    let previous-root-length root-length ;; Record the previous root length for later use
    ;; Code that follows is condensed to make it generic for all crop types
    ;; Call up the daily root and potential regrowth values for this crop type.    
    ;; Generates command like: "set daily-root-value daily-______-root-length"
    let daily-root-sentence ( word "set daily-root-value daily-" land-cover "-root-length" )
    run daily-root-sentence
    ;; Generates command like: "set daily-root-regrowth-value daily-potential-root-regrowth-pasture"
    let daily-root-regrowth-sentence ( word "set daily-root-regrowth-value daily-potential-root-regrowth-" land-cover )
    run daily-root-regrowth-sentence
    
    if print-roots? [
     print ""
     type "ticks = " print ticks type "; " print self 
     type "land-cover = " print land-cover
     type "daily-root-value = " print daily-root-value
     type "roots-damaged? = " print roots-damaged?
     type "root-length = " print root-length
     type "root-bottom before = " print root-bottom
     type "( elevation - groundwater-top ) = " print  ( elevation - groundwater-top )
    ]
    
    ;; Now, update the root length. If undamaged, the roots adopt the new daily value. If the roots descend into the groundwater, then they are now damaged and the length in the groundwater dies.
    ;; If damaged, then the roots could regrow.
    ifelse roots-damaged? = false [ 
      ;; Undamaged roots update their length to the value from the input file
      ifelse daily-root-value > ( elevation - groundwater-top ) [ ;; Soil column above the groundwater
        ;;; Meaning that the area above the groundwater is smaller than the potential root area, then the root length should be equal to height above the water
        set root-length precision ( elevation - groundwater-top ) 4
        ;; Setting as damaged
        set roots-damaged? true
      ]
      [
        ;; Otherwise, there is room for the roots, so the roots are the new length
        set root-length precision daily-root-value 4
      ]
    ]
    [ ;; Damaged roots, check for regrowth 
      ifelse daily-root-value = 0 [
        set root-length 0 ] ;; Roots should die off once the growth period is over. This ensure it happens if the roots had been damaged
      [
        ;;; Regrow damaged roots - this value might be 0, meaning there was no regrowth based on the change in root length set in the update-daily-values procedure
        ;;; But check that the new length is not longer than today's input file length.
        set root-length precision ( root-length + daily-root-regrowth-value ) 4
        if root-length > daily-root-value [ set root-length precision daily-root-value 4 ] ;; Ensures that the roots never get longer than they should be
        if root-length > ( elevation - groundwater-top ) [ set root-length precision ( elevation - groundwater-top ) 4 ] ;; Ensures that the roots do not extend into the water table.
      ]
    ]
    
    ;; Apply precision to the root length
    set root-length precision root-length 4
    
    ;; Now, a check for errors
    if root-length < 0 [
      type "ticks = " print ticks
      show self
      print "root length < 0"
      type "daily-root-value = " print daily-root-value
      type "daily-root-regrowth-value = "print daily-root-regrowth-value
      type "roots-damaged = " print roots-damaged?
      type "soil-column-above-groundwater = " print ( elevation - groundwater-top )
      type "root-length = " print root-length
      print ""
    ]
    
    set root-bottom precision ( elevation - root-length ) 4 ;;; The depth of the roots in absolute terms
    if print-roots? [
      type "root-length AFTER update = " print root-length
      type "root-bottom AFTER = " print root-bottom
    ]
    
                                                            ;; Update the root and upper zones if the root length changed
    if precision previous-root-length 4 != precision root-length 4 [
      Update-soil-horizons-after-plant-growth ;; Updates the root ans upper zone soil and water columns
      patch-level-check-for-water-errors
    ]
  ]
end

to Update-soil-horizons-after-plant-growth
  ;; Update the root and upper zone sizes and water amounts
  ;; This is called after plants adopt new root lengths or their root lengths reset to 0 when plants die or are harvested
  
  If print-procedure-in-code? [ print "Update-soil-horizons-after-plant-growth" ]
  
  ;  ;; The root length has changed, so update the root and upper zones.

    
  ifelse ( precision ( elevation - capillary-top ) 4 ) > .3 and ( precision ( elevation - capillary-top ) 4 ) > root-length [ ;; As long as the soil column is at least 0.3 and the root does not go into the CZ...
                                                                            ;; The future bottom will be the elevation minus whichever is larger, the root length or 0.3 m
                                                                            ;; We are subtracting what will be the soil column from the elevation to get the bottom
                                                                            ;; Otherwise, the bottom is the the elevation minus the soil height above the capillary zone
                                                                            ;; We condensed this into an "and" rather than having the soil-column-above-CZ > root-length be
                                                                            ;; a sub condition because the "else" in both cases is set root-zone-soil-column soil-column-above-CZ
    set root-zone-bottom-FUTURE precision ( elevation - ( max (list .3 root-length ) ) ) 4 ;; 
  ]
  [ ;; the else
    ;; (a) soil-column-above-CZ > .3  but soil-column-above-CZ <= root length or (b) soil-column-above-CZ <= .3 (and any root length)
    ;; the soil-column-above-CZ is smaller than the RZ minimum of .3 or the root descends into the CZ
    set root-zone-bottom-FUTURE capillary-top ;; the root bottom will be the same as the capilalry top
  ]  
    
  let change-RZ-soil-column precision ( root-zone-bottom-FUTURE - root-zone-bottom ) 4 ;; Difference between the future and current level
  ;; Negative means roots are longer and thus RZ larger
  ;; Positive means roots shorter and thus UZ shorter
  
  if change-RZ-soil-column != 0 [ 
                                 ;; The zones would remain unchanged with root growth if a) the root length was smaller than the minimum of 0.3 or
                                 ;; b) the before and after growth were in the capillary zone (and thus there was no UZ before or after)
    ifelse change-RZ-soil-column < 0 [
      ;; If the RZ is deeper now, then soil from the UZ moved into the RZ
      ;; Determine % saturation in the UZ, then assume that the soil left the UZ is at that saturation
      let max-UZ-water-column precision ( ( root-zone-bottom - capillary-top ) * medium-pore-specific-yield ) 5 ;; Water column the original UZ soil column can hold, or the height of the medium pores
      let UZ-saturation precision ( upper-zone-water-column / max-UZ-water-column ) 4 ;; % saturation before resizing
      ;; Determine the water that will move using the size of the soil column that moved and its saturation
      let change-UZ-water-column precision ( ( abs change-RZ-soil-column ) * UZ-saturation * medium-pore-specific-yield ) 4 ;; This is the water column that will be transfered
      ;; Now transfer the water
      set upper-zone-water-column precision ( upper-zone-water-column - change-UZ-water-column ) 4
      set root-zone-water-column precision ( root-zone-water-column + change-UZ-water-column ) 4
    ]
    [
      ;; If the RZ bottom is shallower now, then soil from the RZ moved into the UZ
      ;; Determine % saturation in the RZ, then assume that the soil left the RZ is at that saturation
      let max-RZ-water-column precision ( ( elevation - root-zone-bottom ) * medium-pore-specific-yield ) 5 ;; Water column the original RZ soil column can hold, or the height of the medium pores
      let RZ-saturation precision ( root-zone-water-column / max-RZ-water-column ) 4 ;; % saturation before resizing
      ;; Determine the water that will move using the size of the soil column that moved and its saturation
      let change-RZ-water-column precision ( change-RZ-soil-column * RZ-saturation * medium-pore-specific-yield ) 4;; This is the water column that will be transfered
      ;; Now transfer the water
      set root-zone-water-column precision ( root-zone-water-column - change-RZ-water-column ) 4
      set upper-zone-water-column precision ( upper-zone-water-column + change-RZ-water-column ) 4
    ]
    
    ;; Now, adopt the future buffers as the current levels
    set root-zone-bottom precision root-zone-bottom-FUTURE 4
  ] ;; CLOSES if change-RZ-soil-column != 0 [ 
end

to evapotranspiration
  ;; a) determine plant need at the patch level, b) where the roots are, c) how much water is available for ET, d) do the transpiration, e) evaporate water, f) update the water horizons
  
  If print-procedure-in-code? [ print "evapotranspiration" ]
  
  ;; Debugging code
  ;; Sum up the water in the landscape
  
  ;;; Resetting variables used with ET first
  ask patches [
    set ETc 0
    set transpiration-amount  0
    set evaporation-amount 0
    set water-deficit 0
    set daily-unmet-need 0
    set groundwater-water-column-FUTURE groundwater-water-column
  ]

  update-plant-need        ;; Update the daily ETc based on the land cover
  update-root-distribution ;; Determines how much of the roots are in each zone - in % and absolute terms
  update-water-available   ;; Determines how much water is available for plant intake
  transpiration            ;; Remove water from root and capillary zones to meet water demand and record any deficit. If threshold for days with 100% unmet demand reached, plant dies
  evaporation              ;; Water may evaporate from the surface or the root zone or the groundwater
  update-unmet-need        ;; Toggles on and off critical growing periods; if it is a critical growing period for the crop on a patch, then it determines the daily unmet demand and advances days critical
  
  Update-soil-horizons-After-evapotranspiration
  
  ;; Code to help with debugging any errors
end

to update-plant-need
 ;; Update the daily ETc based on the land cover
 
  If print-procedure-in-code? [ print "update-plant-need" ]
 
  ask patches [
    
    ifelse dead? = false [
      ;; If living, the ETc for the patch corresponds to the land cover
      ;; Generates code like: "set ETc precision ( daily-ETo * daily-Kcb-______ ) 4"
      let need-sentence ( word "set ETc precision ( daily-ETo * daily-Kcb-" land-cover " ) 4")
      run need-sentence
    ]
    [ set ETc 0 ] ;; plant is dead
  ]
end

to update-root-distribution
  ;; Determines how much of the roots are in each zone - in % and absolute terms
 
  If print-procedure-in-code? [ print "update-root-distribution" ]
 
  ask patches with [ dead? = false ] [ ;;
    ifelse root-length = 0 [
      ;; There are no roots on the plant this tick, but there could be in the future.
        set root-length-root-zone 0
        set root-length-capillary-zone 0
        set percent-roots-in-root-zone 0
        set percent-roots-in-capillary-zone 0
    ]
    
    [ ;;; Root length > 0
      ifelse root-bottom >= capillary-top [
        ;; This means that root bottom is in the root zone or at the boundary between the capillary and root zones, so the root is only in the root zone
        set root-length-root-zone root-length
        set root-length-capillary-zone 0
        set percent-roots-in-root-zone 1
        set percent-roots-in-capillary-zone 0
      ]
      [
        ;; Means that the root bottom is in capillary zone, so the root is in both zones
        set root-length-root-zone precision ( elevation - root-zone-bottom ) 4
        set root-length-capillary-zone precision ( root-length - root-length-root-zone ) 4
        ;; It is possible that the root actually descends slightly into the water table. Therefore, we do not want the length of the root
        ;; in the capillary zone to be more than the capillary zone height, otherwise, later, more than 100% of the water in the zone would be available for transpiration
        if root-length-capillary-zone > (capillary-top - groundwater-top ) [ set root-length-capillary-zone precision (capillary-top - groundwater-top ) 4 ]
        set percent-roots-in-root-zone precision ( root-length-root-zone / root-length ) 4
        set percent-roots-in-capillary-zone precision ( 1 - percent-roots-in-root-zone ) 4
      ]     
    ]
  ]
end

to update-water-available
  ;;; Determines how much water is available for plant uptake
  
  If print-procedure-in-code? [ print "update-water-available" ]
  
  ask patches [
    ;; Water reachable in the root zone
    let percent-root-water-reachable 0 ;; % of RZ that has roots in it
    ;; making sure there is no division by 0
    ifelse ( elevation - root-zone-bottom ) > 0 [
     ;; There is an root zone...
      set percent-root-water-reachable precision ( root-length-root-zone / ( elevation - root-zone-bottom ) ) 4
    ]
    [
      ;; No root zone, so set to 0
      set percent-root-water-reachable 0
    ]
    
    ;; Water reachable in the capillary zone
    let percent-capillary-water-reachable 0
    ;; Making sure there is no division by 0
    ifelse (capillary-top - groundwater-top ) > 0 [
     ;; There is a capillary zone...
      set percent-capillary-water-reachable precision ( root-length-capillary-zone / (capillary-top - groundwater-top ) ) 4
    ]
    [
      ;; No capillary zone, so set to 0
      set percent-capillary-water-reachable 0
    ]
            
    ;; Now determine how much actually available, or 10% of the water that is reachable
    let root-zone-water precision ( root-zone-water-column * percent-root-water-reachable ) 4 ;; the height of water column in the area the roots reach in the root zone
    let capillary-water precision ( ( (capillary-top - groundwater-top ) * medium-pore-specific-yield) * percent-capillary-water-reachable ) 4 ;; the height of water column in the area the roots reach in the capillary zone
    set water-available-root-zone precision ( root-zone-water * .1 ) 4 ;; the water available for ET is 10% of this amount
    set water-available-capillary-zone precision ( capillary-water * .1 ) 4 ;; the water available for ET is 10% of this amount

    ;; Code to aid in debugging
    if print-water-available-tracking-data? [
      print ""
      show self
      type "root zone soil column = " print ( elevation - root-zone-bottom )
      type "capillary zone soil column = "print (capillary-top - groundwater-top )
      type "root zone water column = " print root-zone-water-column
      type "capillary zone water column = " print (capillary-top - groundwater-top ) * medium-pore-specific-yield
      type "root length = " print root-length 
      type "root in RZ = " print root-length-root-zone
      type "root in capillary zone = " print root-length-capillary-zone
      type "percent-root-water-reachable = " print percent-root-water-reachable
      type "percent-capillary-water-reachable = " print percent-capillary-water-reachable
      type "root-zone-water-column * percent-root-water-reachable = " print root-zone-water
      type "capillary-water-column * percent-capillary-water-reachable = " print capillary-water
      type "water-available-root-zone = " print water-available-root-zone
      type "water-available-capillary-zone = " print water-available-capillary-zone
      print ""
    ]
  ]   
end

to transpiration
  ;; Remove water from root and capillary zones to meet water demand and record any deficit. If threshold for days with 100% unmet demand reached, plant dies
  
  If print-procedure-in-code? [ print "transpiration" ]
  
  set error-code "transpiration"
  if Print-Update-soil-horizons-after-evapotranspiration? [ ask patch 0 0 [ type "ETc = " print ETc ] ]
  ask patches with [ dead? = false and ETc > 0 ] [ ;; Only withdraw water from living patches with ETc
    
;      if plant-death-message? and year = 22 and day > 320 and day < 331 [
;    if pxcor = 0 and pycor = 0 [
;      type "ticks " print ticks
;      type "day " print day
;      type "root-length = " print root-length
;      type "ETc = " print ETc
;      type "root-zone-water-column = " print root-zone-water-column
;      type "water-available-root-zone = " print water-available-root-zone
;      type "water-available-capillary-zone = " print water-available-capillary-zone
;      print ""
;    ]
;  ]
    
    ;;; Setting a local variable to record the unmet ETc. Starts at all unmet and goes down as water is withdrawn
    let remaining-ETc precision ETc 4
    if print-transpiration? [ ;; Debugging code
      print ""
      show self
      type "starting ETc = "print remaining-ETc
    ]
    
    ;;; Withdrawing water from the root zone first
    let ETc-root precision ( remaining-ETc * percent-roots-in-root-zone ) 4 ;; How much of the ETc the plant will attempt to take from the root zone    
    if print-transpiration? [ ;; Debugging message
      type "water-available-root-zone = " print water-available-root-zone
      type "water-available-root-zone - ETc-root = " print precision ( water-available-root-zone - ETc-root ) 4
      type "root-zone-water-column before transpiration = " print root-zone-water-column
    ]
    ifelse precision ( water-available-root-zone - ETc-root ) 4 > 0 [
      ;;; If there is enough water in the root zone to meet the demand there
      set remaining-ETc precision ( remaining-ETc - ETc-root ) 4 ;; Subtract he full amount needed from the root zone from the overall need
      set root-zone-water-column precision ( root-zone-water-column - ETc-root ) 4 ;; Withdraw the water from the root zone
      set days-no-water 0 ;; Setting the days with no water to 0
      if print-transpiration? [ print "should be enough water in RZ to meet ETc" ] ;; Debugging message
    ]
    [ ;; Not enough water available in the root zone to meet need, so withdraw the partial amount
      set remaining-ETc precision ( remaining-ETc - water-available-root-zone ) 4
      set root-zone-water-column precision ( root-zone-water-column - water-available-root-zone ) 4
       if print-transpiration? [ print "not enough water in RZ to meet ETc" ] ;; Debugging message
    ]      
    if print-transpiration? [ ;; Debugging message
      type "ETc-root = " print ETc-root
      type "ETc after root zone water withdrawn = " print remaining-ETc
      type "RZ water column after root zone water withdrawn = " print root-zone-water-column
    ]
    
    ;;; Capillary zone
    ;; Only do if there is still remaining ETc...
    if remaining-ETc > 0 [
      ifelse precision ( water-available-capillary-zone - remaining-ETc ) 4 > 0 [
        ;;; means that there is need for some water to be taken from the capillary zone AND that
        ;;; there is enough water available in the capillary zone to meet this need
        
        ;; Note: it does not mean that the capillary zone does not compensate for any unmet need from the root zone.
        ;; The amount of water that can be taken is based on the roots and the water available, so the actual amount that compensates for unmet demand in the root zone will likely just be superficial
        ;; Discussed on 11/11/13 with Federico and Moira. Federico confirmed that this coding is correct this way.
        if Print-Update-soil-horizons-after-evapotranspiration? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE ] ]
        set groundwater-water-column-FUTURE precision ( groundwater-water-column-FUTURE - remaining-ETc ) 4 ;;
        if Print-Update-soil-horizons-after-evapotranspiration? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE ] ]
        set remaining-ETc 0 ;; the full amount needed from the capillary zone
        set days-no-water 0 ;; Setting the days with no water to 0
              
      ]
      [
        ;;; Withdraw any amount from the capillary zone that is available, but there is not enough water to meet the full demand
        if Print-Update-soil-horizons-after-evapotranspiration? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE ] ]
        set groundwater-water-column-FUTURE precision ( groundwater-water-column-FUTURE - water-available-capillary-zone ) 4 
        if Print-Update-soil-horizons-after-evapotranspiration? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE ] ]
        set remaining-ETc precision ( remaining-ETc - water-available-capillary-zone ) 4
        ;; updating to indicate the plant is stressed and by how much, since not all of the ETc was met.
        set water-deficit precision remaining-ETc 4
        set stressed? true
        ifelse precision remaining-ETc 4 = precision ETc 4 [
          ;; 100% unmet demand! The plant will advance a counter for consecutive days with 100% unmet demand
          set days-no-water days-no-water + 1
            
          if days-no-water > Days-no-water-death and ( land-cover != "pasture" and land-cover != "fallow" ) [
            ;; If the tolerance for days with no water is reached, the plant dies, except for pasture, which never dies.
            set dead? true
            set root-length 0
            Update-soil-horizons-after-plant-growth
            
            if plant-death-message? [
              if pxcor = 0 and pycor = 0 [
                type land-cover type " crop dead and ETc = " print ETc
                type "    Year " type year type "; day " type day type "; ticks " print day
                type "    water-available-capillary-zone = " type water-available-capillary-zone type "; water-available-root-zone = " type water-available-root-zone type " root-zone-water-column = " print root-zone-water-column
                type "    soil-column-above-CZ = " type ( elevation - capillary-top ) type "; root-length = " print root-length
                print ""
              ]
            ]
          ]
        ]
        [
          set days-no-water 0 ;; Setting the days with no water to 0 
        ]
      ]
      if print-transpiration? [ type "remaining-ETc after capillary zone = " print remaining-ETc ] ;; Debugging message
    ]
    ;; Indicating if there is an error, since this should never be negative
    if remaining-ETc < 0 [
      print "error- remaining ETc negative "
      stop
    ]
    
    ;; Record the final amount that transpired
    set transpiration-amount precision ( ETc - remaining-ETc ) 4
    if print-transpiration? [ type "transpiration-amount = " print transpiration-amount ] ;; debugging message
    ;; Indicating if there is an error, since this should never be negative
    if transpiration-amount < 0 [
      print ""
      print "NEGATIVE TRANSPIRATION"
      show self
      type "ETc = " print ETc
      type "remaining-ETc = " print remaining-ETc
      type "transpiration-amount = " print transpiration-amount      
    ]
  ]
end

to evaporation
  ;;; Water may evaporate from the surface or the root zone or the groundwater
  
  If print-procedure-in-code? [ print "evaporation" ]
  
  set error-code "evaporation" ;; used in debugging
  
  ask patches [
    ;;; Setting the potential evaporation amount based on how much has transpired (subtract the transpired water amount from the ETo)
    set evaporation-amount precision ( daily-ETo - transpiration-amount ) 4
    if Print-Update-soil-horizons-after-evapotranspiration? [ if pxcor = 0 and pycor = 0 [ type "evaporation-amount = " print evaporation-amount ] ]
    ;;; For some cases, Kcb may be higher than 1
    ;;; (e.g., for soybean at the middle of the cropping cycle). In this case, evaporation is 0. In other words, if transpiration is ≥ to ETo, then Evaporation is 0.
    if evaporation-amount < 0 [ set evaporation-amount 0 ]
    
    ;; Actual evapration amount is adjusted later in this procedure to reflect the actual water that evaporates to reflect if the groundwater is at the surface or there is not enough water to evaporate the full amount
    ;;; later, if speed is an issue with a large landscape, we could add back in a check saying only to do evaporation if potential-evaporation-amount > 0 since at certain times of the year with soy only, there might be no evaporation if transpiration takes up all ETo
  
    ifelse groundwater-top = elevation [
      ;;; Meaning that the water table is at the surface and thus the surface is probably flooded.
      ;;; Water will evaporate first from surface water, then if not enough water there to meet PE, the remaining water will come from the groundwater
      ;;; 1.2x the normal amount will evaporate due to tank conditions of surface water
      set evaporation-amount precision ( evaporation-amount * 1.2 ) 4
      
      ifelse surface-water-column - evaporation-amount >= 0 [
        ;;; Enough water on the surface for all the water to come from surface water
        set surface-water-column precision ( surface-water-column - evaporation-amount ) 4
      ]
      [
        ;;; Water will come from the surface and groundwater
        ;;; Tracking how much water removed overall from both areas, so setting a local variable
        let remaining-evaporation-amount precision evaporation-amount 4

        ;;; removing all of the water on the surface
        set remaining-evaporation-amount precision ( remaining-evaporation-amount - surface-water-column ) 4
        if print-evaporation-tracking-data? [ type "surface-water-column before evaporation = " print surface-water-column ]
        set surface-water-column 0
        if print-evaporation-tracking-data? [ type "surface-water-column after evaporation = " print surface-water-column ]
        ;;; the remainder of the amount adjusted for tank evaporation is removed
        if Print-Update-soil-horizons-after-evapotranspiration? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE ] ]
        set groundwater-water-column-FUTURE precision ( groundwater-water-column-FUTURE - remaining-evaporation-amount ) 4
        if Print-Update-soil-horizons-after-evapotranspiration? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE ] ]
      ]
    ]
    [ ;; Groundwater was not at the surface
      ifelse precision capillary-top 4 = precision elevation 4 [
        ;;; Meaning that the top of the capillary zone is at the surface- there is no root zone.
        ;;; Thus water that evaporates will come from the groundwater
        if print-evaporation-tracking-data? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE before evapo = " print groundwater-water-column-FUTURE ] ]
        set groundwater-water-column-FUTURE precision ( groundwater-water-column-FUTURE - evaporation-amount ) 4 ;; Buffer for future groundwater column
        if print-evaporation-tracking-data? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE after evapo = " print groundwater-water-column-FUTURE ] ]
      ]
      [
        ;;; No capillary at the surface, there is a root zone
        let percent-full 0 ;; Local variable for the % fullness of the pores in the root zone
        if print-evaporation-tracking-data? [ ;; Debugging message
          show self
          print land-cover
          type "root-zone-soil-column = " print ( elevation - root-zone-bottom )
          type "root-zone-water-column = " print root-zone-water-column
          type "max water volume = " print ( medium-pore-specific-yield * ( elevation - root-zone-bottom ) )
        ]
        
        ;; There is a special factor that affects evaporation: the drier the soil, the less the evaporation
        if ( elevation - root-zone-bottom ) > 0 [ set percent-full precision ( root-zone-water-column / ( medium-pore-specific-yield * ( elevation - root-zone-bottom ) ) ) 5 ]
        if print-evaporation-tracking-data? [ type "% full = " print percent-full * 100 ] ;; Debugging message
        let RZ-evaporation precision ( evaporation-amount * ( percent-full * percent-full ) ) 4
        if print-evaporation-tracking-data? [ print RZ-evaporation ] ; Debugging message
        
        ifelse precision ( root-zone-water-column - RZ-evaporation ) 4 >= 0 [
          ;; Enough water for full PE amount from the RZ
          set evaporation-amount RZ-evaporation
          if print-evaporation-tracking-data? [ if pxcor = 0 and pycor = 0 [ type "root-zone-water-column b4 evapo= " print root-zone-water-column ] ]
          set root-zone-water-column precision ( root-zone-water-column - RZ-evaporation ) 4
          if print-evaporation-tracking-data? [ if pxcor = 0 and pycor = 0 [ type "root-zone-water-column b4 after= " print root-zone-water-column ] ]
        ]
        [
          ;;; Not enough water to evaporate all PE from the RZ so depending on conditions, water may also evaporate from the groundwater
          
          ;; Setting a local variable keeping track of how of the PE remains
          let remaining-evaporation-amount precision evaporation-amount 4 

          ;;; All water in the RZ evaporates
          set remaining-evaporation-amount precision ( remaining-evaporation-amount - root-zone-water-column ) 4
          set evaporation-amount precision root-zone-water-column 4
          set root-zone-water-column 0
          
          ;;; If the RZ soil column is smaller than .2 m because CZ is close to surface, the remaining PE evaporates from the capillary zone (groundwater)
          if precision ( elevation - root-zone-bottom ) 4 < 0.2 [
            if print-evaporation-tracking-data? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE b4 evapo= " print groundwater-water-column-FUTURE ] ]
            set groundwater-water-column-FUTURE precision ( groundwater-water-column-FUTURE - remaining-evaporation-amount ) 4 ;; all remaining need will be met due to capillary action
            if print-evaporation-tracking-data? [ if pxcor = 0 and pycor = 0 [ type "groundwater-water-column-FUTURE after evapo = " print groundwater-water-column-FUTURE ] ]
            set evaporation-amount precision ( evaporation-amount + remaining-evaporation-amount ) 4 
          ]
        ];; end of condition when not enough water in RZ to evaporate
      ];;end of condition with no GW at the surface      
    ]   
    if print-evaporation-tracking-data? [ ;; debugging message
      type "daily-ETo = " print daily-ETo
      type "transpiration-amount = " print transpiration-amount
      type "initial evaporation-amount before adjustments (daily-ETo - transpiration-amount) = " print daily-ETo - transpiration-amount
      if groundwater-top = elevation [
        print "groundwater-top = elevation, so evaporation amount will be higher (x1.2)"
      ]
      type "evaporation-amount = " print evaporation-amount
      print ""
    ]
  ] 
end

to update-unmet-need
  ;; Toggles on and off critical growing periods; if it is a critical growing period for the crop on a patch, then it determines the daily unmet demand and advances days critical
  
  ; Daily-unmet-need is from 0 to 1 where 0 is completely met need and 1 is completely unmet need
  ; Cumulative-percent-daily-unmet-need with 0 meant that there was never unmet need, higher means some unmet need
  
  If print-procedure-in-code? [ print "update-unmet-need" ]
  
  ;;; Toggle on and off the critical growing periods for each crop
  ;; Except for pasture, which is always critical
  let crop-list-except-pasture remove "pasture" crop-list ;; remove pasture from this list
  ( foreach crop-list-except-pasture [ ;; Repeat for each crop
    ;; Generates code like: "if day = critical-grow-______-start [ set critical-growing-period-____? true ] "
    let critical-start-sentence ( word "if day = critical-grow-" ? "-start [ set critical-growing-period-" ? "? true ] " )
    run critical-start-sentence
    ;; Generates code like: "if day = critical-grow-______-end [ set critical-growing-period-______? false ] "
    let critical-end-sentence ( word "if day = critical-grow-" ? "-end [ set critical-growing-period-" ? "? false ] " )
    run critical-end-sentence
  ] )
  
  ( foreach crop-list [
    ;; Generic code for all crop types to update
    ;; Generates code like: "set critical-now? critical-growing-period-_______?"
    let critical-now-sentence ( word "set critical-now? critical-growing-period-" ? "?" )
    run critical-now-sentence
    
    if critical-now? = true [
      ask patches with [ land-cover = ? and dead? = false ] [
        ;; Adding this day's % unmet need to a variable tracking the cumulative % unmet need in the critical growing period
        set daily-unmet-need precision ( water-deficit / ETc ) 4 ;; % unmet need
        if daily-unmet-need > 1 [ set daily-unmet-need 1 ]
        set cumulative-percent-daily-unmet-need precision ( cumulative-percent-daily-unmet-need + daily-unmet-need ) 4 ;; Cumulative unmet need during the critical period
        set days-current-crop-critical days-current-crop-critical + 1 ;; Days this crop is critical
      ]
    ]
  ] )
  
  if print-unmet-need? [ ;; For debugging
    ask patches [
      print ""
      show self
      type "land-cover = " print land-cover
      type "ETc = " print ETc
      type "water-deficit = " print water-deficit
      type "daily-unmet-need = " print daily-unmet-need
      type "cumulative-percent-daily-unmet-need = " print cumulative-percent-daily-unmet-need
      type "surface-water-column = " print surface-water-column
    ]
  ]
end

to Update-soil-horizons-after-evapotranspiration
  ;; If the groundwater water column changed due to evaporation or transpiration, then resize the groundwater and update the size and water content of the zones above it
  
  If print-procedure-in-code? [ print "Update-soil-horizons-after-evapotranspiration" ]
  
  ask patches [
    let capillary-recharge precision ( groundwater-water-column-FUTURE - groundwater-water-column ) 4 ;;
    let GW-top-decrease precision ( capillary-recharge / large-pore-specific-yield ) 4
    set groundwater-top-FUTURE precision ( groundwater-top + GW-top-decrease ) 4 ;; Convert the GW water to height
    set groundwater-water-column-FUTURE precision ( ( groundwater-top-FUTURE - soil-bottom ) * groundwater-specific-yield ) 4
    
;    set groundwater-top-FUTURE precision ( ( groundwater-water-column-FUTURE / groundwater-specific-yield ) + soil-bottom  ) 4 ;; Convert the GW water to height
;    
    if precision groundwater-top-FUTURE 4 != precision groundwater-top 4 [ ;; The groundwater top changed, so the size of the zones and water amounts will need to be updated

      
      set error-code "evapotranspiration - BEFORE updating horizons"
      patch-level-check-for-water-errors
     
     ;; If the GW top changed after to evapotranspiration, it is only possible that it decreases, so we can assume that some soil that was in the GW/capillary zones
     ;; will now be in the root or upper zones
      
      ;; Next, determine the CAPILLARY TOP and then the size of the capillary zone soil column
      ;;; The capillary zone is the 0.8 m of soil above the water table unless the elevation (surface) is closer
      set capillary-top-FUTURE precision ( groundwater-top-FUTURE + 0.8 ) 4
      if capillary-top-FUTURE > elevation [ set capillary-top-FUTURE elevation ] ;; If the capillary top would be higher than the elevation, reset it to be the elevation
      
      ;; Next, determine the future ROOT BOTTOM
      ifelse ( precision ( elevation - capillary-top-FUTURE ) 4 ) > .3 and ( precision ( elevation - capillary-top-FUTURE ) 4 ) > root-length [ ;; As long as the soil column is at least .3 and the root does not go into the CZ...
                                                                              ;; We condensed this into an "and" rather than having the soil-column-above-CZ > root-length be
                                                                              ;; a sub condition because the "else" in both cases is set root-zone-soil-column soil-column-above-CZ
      set root-zone-bottom-FUTURE precision ( elevation - ( max (list .3 root-length ) ) ) 4 ;; the depth is the elevation minuwhichever is larger, the root length or .3 m
                                                                                             ;     set   root-zone-soil-column-FUTURE precision ( max (list .3 root-length ) ) 4 ;; the RZ soil column is whichever is larger, the root length or .3 m
      ]
      [ ;; the else
        ;; (a) soil-column-above-CZ > .3  but soil-column-above-CZ <= root length or (b) soil-column-above-CZ <= .3 (and any root length)
        ;; the soil-column-above-CZ is smaller than the RZ minimum of .3 or the root descends into the CZ
        set root-zone-bottom-FUTURE precision ( elevation - ( elevation - capillary-top-FUTURE ) ) 4 ;; the depth is the elevation minus the soil height above the CZ top
                                                                                     ;     set   root-zone-soil-column-FUTURE precision soil-column-above-CZ 4 ;; the RZ is any height above the CZ
      ]
      
      ;; Update the root and upper zone water columns
      Update-RZ-UZ-after-GW-went-down
      
      ;; Adopt the butter layers as the new values
      
      if Print-Update-soil-horizons-after-evapotranspiration? [
       if pxcor = 0 and pycor = 0 [
         print ""
         type "old groundwater-top = " print groundwater-top
         type "groundwater-top-FUTURE = " print groundwater-top-FUTURE
         type "old groundwater-water-column = " print groundwater-water-column
         type "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE
         type "evaporation-amount = " print evaporation-amount
         type "transpiration-amount = " print transpiration-amount
       ]
      ]
      set groundwater-top groundwater-top-FUTURE
      set groundwater-water-column precision groundwater-water-column-FUTURE 4
      set capillary-top capillary-top-FUTURE
      set root-zone-bottom root-zone-bottom-FUTURE
    
      set error-code "evapotranspiration - after updating horizons"
      patch-level-check-for-water-errors
    ]
  ]
  set error-code "evapotranspiration" 
end

to Update-RZ-UZ-after-GW-went-down
  
  If print-procedure-in-code? [ print "Update-RZ-UZ-after-GW-went-down" ]
  
  if print-Update-RZ-UZ-after-GW-went-down? [
    print ""
    print "GW went down - updating horizons"
    show self
    type "elevation = "print elevation
    type "groundwater-top = " print groundwater-top
    type "groundwater-top-FUTURE = " print groundwater-top-FUTURE
    type "capillary-top = " print capillary-top
    type "capillary-top-FUTURE = " print capillary-top-FUTURE
    type "starting root-zone-soil-column = " print elevation - root-zone-bottom
    type "starting upper-zone-soil-column = " print ( root-zone-bottom - capillary-top )
    type "starting root-zone-bottom = " print root-zone-bottom
    type "root-zone-bottom-FUTURE = " print root-zone-bottom-FUTURE
    type "root-length = " print root-length
  ]
  
  let change-GW-top precision ( groundwater-top-FUTURE - groundwater-top ) 4 ;; Change = new - old ; here, this will always be negative since the GW can only decrease due to evapotranspiration    
  let change-GW-water-column precision ( change-GW-top * medium-pore-specific-yield ) 4 ;; The change to the GW water column, or the water that will move to the UZ or RZ.
                                                                        ;; Multiplying by medium pore specific yield because the medium pores
                                                                        ;; in the soil that leaves the GW zone will be fully saturated
;  set groundwater-water-column-FUTURE groundwater-water-column-FUTURE - ( abs change-GW-water-column )
  
  ;; TO DO... if the change-GW-top was more than 0.8 then there is potentially a problem since our assumptions about water fullness would need to be updated
  
                                                                        ;; Next, update the ROOT and UPPER ZONE SOIL COLUMNS. At least one of them will now be LARGER.
  let root-zone-soil-column-FUTURE precision ( elevation - root-zone-bottom-FUTURE ) 4
  let upper-zone-soil-column-FUTURE precision ( root-zone-bottom-FUTURE - capillary-top-FUTURE ) 4;; The UZ will be whatever soil height above the CZ is not in the RZ  
  if print-Update-RZ-UZ-after-GW-went-down? [
    type "change RZ SC = " print root-zone-soil-column-FUTURE - ( elevation - root-zone-bottom )
    type "change UZ SC = " print upper-zone-soil-column-FUTURE - ( root-zone-bottom - capillary-top )
  ]
                                                                                                 ;; UPDATE RZ WATER   
                                                                                                 ;; Add the water that left the groundwater to the RZ in proportion to the change in the soil column, and then any remaining wwater to the UZ
                                                                                                 ;; What % of the water that left the GW will go to the RZ.
  if print-Update-RZ-UZ-after-GW-went-down? [
    type "change-GW-top = " print change-GW-top
    type "change-GW-water-column = " print change-GW-water-column
    type "root-zone-soil-column-FUTURE = " print root-zone-soil-column-FUTURE
    type "upper-zone-soil-column-FUTURE = " print upper-zone-soil-column-FUTURE
  ]
  let change-RZ-soil-column precision ( root-zone-bottom-FUTURE - root-zone-bottom ) 4 ;; Change to the RZ soil column
  let proportion-GW-soil-to-RZ precision ( change-RZ-soil-column / ( abs change-GW-top ) ) 4;; % of the soil that left the GW that went to the RZ; USING ABS SINCE the GW CHANGE IS NEGATIVE
                                                                             ;; ALSO the % of the GW water that went to the RZ, due to uniform soil saturation
  ;;;; NOTE water that moved to the UZ is not merely just the water that left the GW - the water that moved to the RZ
  ;; There are situations when the GW top was near the surface and there was a capillary zone, but the CZ was less than the 0.8 m minimum for the capillary zone
  ;; SO after the update, some of the height of the soil that lef the GW is now capillary and some is RZ but none is UZ
  ;; e.g: elevation is 0; old GW top was -0.75; old capillary top was 0;; NEW GW top is -0.85; new CT is -0.05; new RZ bottom is -0.05, so the RZ soil column is 0.05 m and there is no UZ.
  let change-UZ-soil-column precision ( precision ( root-zone-bottom-FUTURE - capillary-top-FUTURE ) 4 - precision ( root-zone-bottom - capillary-top ) 4 ) 4;; Change to the RZ soil column
  let proportion-GW-soil-to-UZ precision ( change-UZ-soil-column / ( abs change-GW-top ) ) 4;; % of the soil that left the GW that went to the UZ; USING ABS SINCE the GW CHANGE IS NEGATIVE
                                                                             ;; ALSO the % of the GW water that went to the UZ, due to uniform soil saturation
  let new-RZ-water precision ( abs ( change-GW-water-column * proportion-GW-soil-to-RZ ) ) 4 ;; USING ABS BECAUSE THE CHANGE TO THE GW IS NEGATIVE
  let new-UZ-water precision ( abs ( change-GW-water-column * proportion-GW-soil-to-UZ ) ) 4 ;; USING ABS BECAUSE THE CHANGE TO THE GW IS NEGATIVE
  
                                                                                 ;; Next, add the water to the water columns
  set root-zone-water-column precision ( root-zone-water-column + new-RZ-water) 4
  set upper-zone-water-column precision ( upper-zone-water-column + new-UZ-water ) 4
  
  if print-Update-RZ-UZ-after-GW-went-down? [
    type "change-RZ-soil-column = " print change-RZ-soil-column
    type "proportion-GW-soil-to-RZ = " print proportion-GW-soil-to-RZ
    type "new-RZ-water = " print new-RZ-water
    type "new-UZ-water = " print new-UZ-water
    type "new root-zone-water-column = " print root-zone-water-column
    type "new upper-zone-water-column = " print upper-zone-water-column
  ]
  
end

to harvest
  ;; Updated land cover after previous day's harvest, calculates the crop yield factor and yield for patches when their harvest day is the current day
  
  If print-procedure-in-code? [ print "harvest" ]
  
  ;; The annual yield is only > 0 if there was a harvest the previous day.
  ;; This updates the land cover to fallow and resets the yield tracking variables
  ;; We do this the day after the harvest to ensure that the land cover and yield values are recorded properly if data is exported at the end of the run
  ask patches with [ is-number? annual-yield and land-cover != "pasture" ] [
    plant-fallow   
    set annual-yield " "
    set crop-yield-factor 0
    set cumulative-percent-daily-unmet-need 0
    set days-current-crop-critical 0
    set root-length 0
    Update-soil-horizons-after-plant-growth
    set roots-damaged? true
  ]
  
  ask patches with [ day-harvest = day ] [

    ;; Setting the Crop yield factor.
    ;; Crop yield factor is relates directly to unmet water demand during critical growth periods and used to determine yields.
    ;; The factor is scaled from 0 (full potential output) to 1 (no output)    
    ifelse dead? = true [
      set crop-yield-factor 1 ;; No output if dead
    ]
    [
      ;; Crop living, so there will be a yield. Calculate the crop yield factor based on the unmet yield of critical days
      set crop-yield-factor precision ( cumulative-percent-daily-unmet-need / days-current-crop-critical ) 4
    ]
  
    ;; Determine yield for this cycle only
    ;; generates: "set annual-yield ( 1 - crop-yield-factor ) * ____-tonnage-per-patch"
    let yield-sentence ( word "set annual-yield precision ( ( 1 - crop-yield-factor ) * " land-cover "-tonnage-per-patch ) 4" )
    run yield-sentence 

    ;; Add this annual yield to the patch's cumulative output for this crop
    ;; generates: "set patch-_____-output patch-____-output + annual-yield"
    let output-sentence ( word "set patch-" land-cover "-output precision ( patch-" land-cover "-output + annual-yield ) 4 " )      
    run output-sentence 

    ;; Reset cropping variables      
    if rotation-year > 3 [ set rotation-year 1 ] ;; Reset the rotation if it was above 3
    set day-harvest ""
    if land-cover != "pasture" [ ;; Do not reset these for pasture only since we assume pasture remains in the same state it was when the year changes
      set dead? true
    ]
  ]
end

to rain-infiltration
  ;; Determines how much of precipitation will attempt to attempt to infiltrate immediately and infiltrate the new water and any previous water n the surface. Adds runoff to the surface at the end of the procedure.
  
  If print-procedure-in-code? [ print "rain-infiltration" ]
  
  set Landscape-cumulative-water-height precision ( Landscape-cumulative-water-height + ( ( count patches ) * daily-rain-amount ) ) 4 ;; Adding the rain to the total water created
  set error-code "infiltration"

  ;; Determine runoff and infiltration amounts
  ;; Two options for the % runoff: fixed runoff or random
  let runoff-percent 0 ;; Local variable
  ifelse Fixed-runoff? [
    set runoff-percent Fixed-runoff-amount ;; Set by slider. Will not vary within one simulation
  ]
  [
    set runoff-percent ( random 70 ) + 3 ;; Random uniform between 3 and 72, inclusive. 
  ]
  set runoff-percent precision ( runoff-percent / 100 ) 4
  ;; Determine the actual runoff amount
  let runoff precision ( runoff-percent * daily-rain-amount ) 4
  ;; Determine how much will remain and infiltrate now
  let new-water precision ( daily-rain-amount - runoff ) 4 ;; rain - runoff

  ask patches [
    ;; First add the new water from precipitation (non-runoff amount) to the surface
    set surface-water-column precision ( surface-water-column + new-water ) 4
    
    ;; if there is surface water, dur to rain or any water previously on the surface, attempt to infiltrate it now
    if surface-water-column > 0 [
      infiltration
      patch-level-check-for-water-errors
    ]
    ;; Now add the runoff to the surface
    set surface-water-column precision ( surface-water-column + runoff ) 4
  ]
  
end


to infiltration
  ;; Attempt to infiltrate water on the surface. Start from the top and assume water flows downward if it fills the pores in the zone it is in. Possibly adjust water horizons based don infiltration.
  
  If print-procedure-in-code? [ print "infiltration" ]
  
  ;; Setup a future vaalue for the groundwater-top-FUTURE 
  
  set groundwater-top-FUTURE groundwater-top
  set groundwater-water-column-FUTURE groundwater-water-column
  
  if print-rain-infiltration? [ ;; Debugging code
      
      print ""
      type self type " ticks = " show ticks
      print "starting values"
      type "surface-water-column = " print surface-water-column
      type "elevation = " print elevation
      type "groundwater-top-FUTURE = " print groundwater-top-FUTURE
      type "capillary-top = " print capillary-top
      type "root-zone-bottom = " print root-zone-bottom
      type "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE
      type "root-zone-water-column = " print root-zone-water-column
      type "upper-zone-water-column = " print upper-zone-water-column
    ]
  
  ;; Start with the root zone
  ifelse root-zone-bottom < elevation [ ;; If there is a root zone...
                                     ;;; Determine the max possible amount of water in the root zone
    let max-RZ-water-column precision ( ( elevation - root-zone-bottom )  * medium-pore-specific-yield ) 4
    
    if print-rain-infiltration? [ ;; Debugging code
      print ""
      show self
      print "There is a root zone with starting fullness = " print ( (root-zone-water-column / max-RZ-water-column ) * 100 )
    ]
    ;; Add the surface water to the root zone
    set root-zone-water-column precision ( root-zone-water-column + surface-water-column ) 4
    
    if print-rain-infiltration? [ ;; Debugging code
      type "root-zone-water-column after adding water = " print root-zone-water-column
    ]
    
    set surface-water-column 0
    ;; Determine if there is any excess water from the root zone
    let excess-from-root-zone precision ( root-zone-water-column - max-RZ-water-column ) 4
    if print-rain-infiltration? [ type "excess-from-root-zone = " print excess-from-root-zone ]
    ;;; If there is more water than the medium pores in the RZ can handle, the excess will try to go into the UZ
    ifelse excess-from-root-zone > 0 [ 
      set root-zone-water-column precision max-RZ-water-column 4 ;; Reset the RZ water column to the maximum amount
      
                                                                   ;; Have the excess go to the UZ if there is room, or if not, to groundwater.
      ifelse root-zone-bottom > capillary-top [ ;; If there is an upper zone...        
                                          ;; Determine how much room is available in medium pores for the new water
                                          
        let max-UZ-water-column precision ( ( root-zone-bottom - capillary-top ) * medium-pore-specific-yield ) 4
        
        if print-rain-infiltration? [ ;; Debugging code
          print ""
          type "there is a UZ with starting fullness = " print ( (upper-zone-water-column / max-UZ-water-column ) * 100 )
        ]
        
        ;; Add the new water to the upper zone
        set upper-zone-water-column precision ( upper-zone-water-column + excess-from-root-zone ) 4
        ;; Determine if there is too much water in the UZ and any excess
        let excess-from-upper-zone precision ( upper-zone-water-column - max-UZ-water-column ) 4
        
        if print-rain-infiltration? [ ;; Debugging code
          type "upper-zone-water-column after adding all excess water = " print upper-zone-water-column
        ]
        
        ;; If there is more water than the medium pores can handle, so some will go down to water table
        ifelse excess-from-upper-zone > 0 [
          if print-rain-infiltration? [ type "excess-from-upper-zone = " print excess-from-upper-zone ]
          set upper-zone-water-column precision max-UZ-water-column 4 ;; Reset UZ water to maximum
          set groundwater-top-FUTURE precision ( groundwater-top-FUTURE  + ( excess-from-upper-zone / large-pore-specific-yield ) ) 4
          ;; Set the new GW water column based on the new top
          set groundwater-water-column-FUTURE precision ( ( groundwater-top-FUTURE - soil-bottom ) * groundwater-specific-yield ) 4
            
          if print-rain-infiltration? [ 
           print "values before going to horizon update:"
           type "GWT will go up by ( excess-from-upper-zone / large-pore-specific-yield )" print ( excess-from-upper-zone / large-pore-specific-yield )
           type "new groundwater-top-FUTURE = " print groundwater-top-FUTURE
          ]
          
          Update-soil-horizons-after-infiltration ;; Will need to resize the GW zone and the zones above it
        
        ]
        [
          ;; If there is no excess, no water goes down to water table - we are keeping this for verrification purposes.
        ]
      ] ;; CLOSES ifelse root-zone-bottom > capillary-top [ ---- ie was there an upper zone
      
      [ ;; This is the else from  ifelse root-zone-bottom > capillary-top
        ;; No UZ, so excess from RZ will go to groundwater if the groundwater is not at the surface
        if print-rain-infiltration? [ type "No UZ so RZ excess goes directly to groundwater = " print excess-from-root-zone ]
        ;;; Water table not at surface, so room for some or all of surface water to go to groundwater, the exact amount that the WT can handle will be determined by the update groundwater top procedure later
        set groundwater-top-FUTURE precision ( groundwater-top-FUTURE + ( excess-from-root-zone / large-pore-specific-yield ) ) 4
        ;; Set the new GW water column based on the new top
        set groundwater-water-column-FUTURE precision ( ( groundwater-top-FUTURE - soil-bottom ) * groundwater-specific-yield ) 4
        
        if print-rain-infiltration? [ 
          print "values before going to horizon update:"
          type "GWT will go up by ( excess-from-root-zone / large-pore-specific-yield )" print ( excess-from-root-zone / large-pore-specific-yield )
          type "new groundwater-top-FUTURE = " print groundwater-top-FUTURE
        ]
        
        Update-soil-horizons-after-infiltration ;; Will need to resize the GW zone and the zones above it
      ]             
      
    ] ;; CLOSES ifelse excess-from-root-zone > 0 [ 
    [ ;; There was no excess from the root zone, so no change to groundwater
    ]
  ] ;; CLOSES ifelse root-zone-bottom < elevation [ ;; If there is a root zone...
  
  [ ;; There was no root zone, so water will attempt to infiltrate the groundwater but if not, stay on the surface
        
    ;; If there is no RZ then there is no upper zone either, so water will go to groundwater directly
    ifelse groundwater-top-FUTURE < elevation [
      if print-rain-infiltration? [ type "No RZ and groundwater top not at surface already, so water will go directly to groundwater"]
      ;; If there is a capillary zone, room for some or all of water to infiltrate. The exact amount that the WT can handle will be determined by the update groundwater top procedure
      set groundwater-top-FUTURE precision ( groundwater-top-FUTURE + ( surface-water-column / large-pore-specific-yield ) ) 4
      ;; Set the new GW water column based on the new top
      set groundwater-water-column-FUTURE precision ( ( groundwater-top-FUTURE - soil-bottom ) * groundwater-specific-yield ) 4
      
      if print-rain-infiltration? [ 
        print "values before going to horizon update:"
        type "GWT will go up by ( surface-water-column / large-pore-specific-yield )" print ( surface-water-column / large-pore-specific-yield )
        type "new groundwater-top-FUTURE = " print groundwater-top-FUTURE
      ]
      set surface-water-column 0
      
      Update-soil-horizons-after-infiltration
    ]
    [
      ;; The water table is at the surface, so no water goes down at all, and instead all water stays on the surface - this is left here for verrification
    ]
  ]
  
  
  if print-rain-infiltration? [ ;; Debugging code
    print ""
    print "After all updating..."
    type "surface-water-column = " print surface-water-column
    type "groundwater-top = " print groundwater-top
    type "capillary-top = " print capillary-top
    type "root-zone-bottom = " print root-zone-bottom
    type "groundwater-water-column = " print groundwater-water-column
    type "root-zone-water-column = " print root-zone-water-column
    type "upper-zone-water-column = " print upper-zone-water-column
  ]
end

to Update-soil-horizons-after-infiltration
  ;; The GW can only go up due to infiltration
  
  If print-procedure-in-code? [ print "Update-soil-horizons-after-infiltration" ]
  
  ;; Just before this procedure is called, the groundwater top is updated with excess water from the root zone, upper zone, or any surface water.
  
  ;; First, check if the new GW top is higher than the elevation, meaning that the groundwater should be at the surface and some water will go back on the surface
  
    
  ifelse groundwater-top-FUTURE > elevation [ ;; If the GW top is higher than the surface, then the water goes back to the surface
                                       ;; This would be unlikely unless there had been a large storm and the RZ UZ had been very small
    ;; Confirmed with Federico that large pore specific yield is correct for here, since we use that specific yield during infiltration to determine how much the GW top future went up
    set surface-water-column precision ( ( groundwater-top-FUTURE - elevation ) * large-pore-specific-yield ) 4 ;; Determine how much water goes back: soil height of gw top above elevation, mulyiply by specific yield   
    set groundwater-top elevation
    set groundwater-water-column precision ( ( groundwater-top - soil-bottom ) * groundwater-specific-yield ) 4
    ;; In this case, the there will be no capillary, root, or upper zones.
    set capillary-top elevation
    set root-zone-bottom elevation
    set upper-zone-water-column 0
    set root-zone-water-column 0
  ]
  [
    ;;; Otherwise, the GW top is not at the surface and there is a capillary zone and possibly and root and upper zones
        
    ;; Determine the CAPILLARY TOP and then the size of the capillary zone soil column
    ;;; The capillary zone is the 0.8 m of soil above the water table unless the elevation (surface) is closer
    set capillary-top-FUTURE precision ( groundwater-top-FUTURE + 0.8 ) 4

    if capillary-top-FUTURE > elevation [ set capillary-top-FUTURE elevation ] ;; If the capillary top would be higher than the elevation, reset it to be the elevation

    ifelse ( precision ( elevation - capillary-top-FUTURE ) 4 )  > .3 and ( precision ( elevation - capillary-top-FUTURE ) 4 ) > root-length [
      ;; ( elevation - capillary-top ) is the height of the soil column above the capillary top
      ;; As long as the soil column is at least 0.3 and the root does not go into the CZ...
      ;; The future bottom will be the elevation minus whichever is larger, the root length or 0.3 m
      ;; We are subtracting what will be the soil column from the elevation to get the bottom
      ;; Otherwise, the bottom is the the elevation minus the soil height above the capillary zone
      ;; We condensed this into an "and" rather than having the soil-column-above-CZ > root-length be
      ;; a sub condition because the "else" in both cases is set root-zone-soil-column soil-column-above-CZ
      set root-zone-bottom-FUTURE precision ( elevation - ( max (list .3 root-length ) ) ) 4 ;; 
    ]
    [ ;; the else
      ;; In this case, there will be no upper zone
      ;; (a) soil-column-above-CZ > .3  but soil-column-above-CZ <= root length or (b) soil-column-above-CZ <= .3 (and any root length)
      ;; the soil-column-above-CZ is smaller than the RZ minimum of .3 or the root descends into the CZ
      set root-zone-bottom-FUTURE capillary-top-FUTURE ;; the root bottom will be the same as the capilalry top
    ]  
    
    ;; Update the root and upper zone water columns
    Update-RZ-UZ-after-GW-went-up
       
    ;; Update the horizons and water columns with the future values
    set groundwater-top groundwater-top-FUTURE
    set groundwater-water-column precision groundwater-water-column-FUTURE 4
    set capillary-top capillary-top-FUTURE
    set root-zone-bottom root-zone-bottom-FUTURE
  ]
end

to Update-RZ-UZ-after-GW-went-up
  ;; Updating the 
  
  If print-procedure-in-code? [ print "Update-RZ-UZ-after-GW-went-up" ]
  
  let current-upper-zone-soil-column precision ( root-zone-bottom - capillary-top ) 4
  if print-Update-RZ-UZ-after-GW-went-up? [
   print ""
   print "GW top increase led to horizon updating"
   type "starting upper-zone-water-column = " print upper-zone-water-column
   type "starting root-zone-water-column = " print root-zone-water-column
  ]
  
  if current-upper-zone-soil-column > 0 [ ;; Check to see if there is an upper zone at all
    let future-upper-zone-soil-column precision ( root-zone-bottom-FUTURE - capillary-top-FUTURE ) 4
    ;; Update the UPPER ZONE WATER
    if precision current-upper-zone-soil-column 4 != precision future-upper-zone-soil-column 4 [
      let change-UZ-soil-column precision ( future-upper-zone-soil-column - current-upper-zone-soil-column ) 4 ;; How much the soil coloumn will change; WILL BE A NEGATIVE NUMBER OR 0
      let max-UZ-water-column precision ( current-upper-zone-soil-column * medium-pore-specific-yield ) 5 ; What is the height of the medium pores, or how much water the soil column can hold
      let UZ-saturation precision ( upper-zone-water-column / max-UZ-water-column ) 4 ;; current % saturation: water / max water
      let UZ-water-loss precision ( change-UZ-soil-column * UZ-saturation * medium-pore-specific-yield ) 4 ;; How much water was in the soil the UZ lost; will be a negative value
      set upper-zone-water-column precision ( upper-zone-water-column + UZ-water-loss ) 4;; The water loss is negative already, so adding it rather than subtracting
                                                                          ;; Not explicitly adding the change to the GW because the movement is implied in the way the new GW top was calculated
      if print-Update-RZ-UZ-after-GW-went-up? [
        type "current-upper-zone-soil-column = " print current-upper-zone-soil-column
        type "future-upper-zone-soil-column = " print future-upper-zone-soil-column
        type "change-UZ-soil-column = " print change-UZ-soil-column
        type "UZ-saturation = " print UZ-saturation
        type "UZ-water-loss = " print UZ-water-loss
        type "ending upper-zone-water-column = " print upper-zone-water-column
      ]
    ]
  ]
  
  let current-root-zone-soil-column precision ( elevation - root-zone-bottom ) 4
  if current-root-zone-soil-column > 0 [ ;; Check to see if there is a root zone at all
    let future-root-zone-soil-column precision ( elevation - root-zone-bottom-FUTURE ) 4
    ;; Update the ROOT ZONE WATER
    if current-root-zone-soil-column != future-root-zone-soil-column [
      let change-RZ-soil-column precision ( future-root-zone-soil-column - current-root-zone-soil-column ) 4 ;; How much the soil coloumn will change; WILL BE A NEGATIVE NUMBER OR 0
      let max-RZ-water-column precision ( current-root-zone-soil-column * medium-pore-specific-yield ) 5 ;; What is the height of the medium pores, or how much water the soil column can hold
      let RZ-saturation precision ( root-zone-water-column / max-RZ-water-column ) 4 ;; current % saturation: water / max water
      let RZ-water-loss precision ( change-RZ-soil-column * RZ-saturation * medium-pore-specific-yield ) 4 ;; How much water was in the soil the RZ lost; will be a negative value
      set root-zone-water-column precision ( root-zone-water-column + RZ-water-loss ) 4 ;; The water loss is negative already, so adding it rather than subtracting
                                                                        ;; Not explicitly adding the change to the GW because the movement is implied in the way the new GW top was calculated
      if print-Update-RZ-UZ-after-GW-went-up? [
        type "future-root-zone-soil-column = " print future-root-zone-soil-column
        type "change-RZ-soil-column = " print change-RZ-soil-column
        type "RZ-water-loss = " print RZ-water-loss
        type "ending root-zone-water-column = " print root-zone-water-column
      ]
    ]
  ]
end

to surface-flow
  ;; Water flow is based on equilibrium levels and the flows repeat to ensure equilibrium across the entire landscape is reached. Flow happen asynchronously.
  
  If print-procedure-in-code? [ print "surface-flow" ]
  
  ;; Debugging code
;  if print-tracking-error? [ set water-before sum [ groundwater-water-column + surface-water-column + upper-zone-water-column + root-zone-water-column ] of patches ]
  
  ;; Note - We assume that we have a small enough landscape that all water can move across the entire landscape to the middle and reach equilibrium.
  ;; This would change if we had a very large landscape
  ;; The number of repeats ensures that water will run across the entire landscape moving one patch per repeat. Water will be at equilibrium at the end.
  ;; Each repeat, a cell will exchange water with its neighbors and update asynchronously.
  repeat ( (abs max-pxcor) + (abs min-pxcor) + 1 ) [
    ;; Start with the low patches and go to high to ensure that water will only travel one patch per repeat. If we had it from high to low, water could flow
    ;; all the way across the landscape in just one repeat
    foreach Landscape-Elevation-list [ ;; Landscape-Elevation-list is sorted from low to high
      ask ? [
          ;; Only calculate flow if this patch or any of its neighbors has water on it
        if surface-water-column > 0 or any? neighbors with [ surface-water-column > 0 ] [
          ;; Create temporary variables for flow code
          ;; Baseline = the center cell or the cell that is comparing itself to its neighbors
          ;; Defining the baseline values to which the neighbors will compare
          let baseline-water-column surface-water-column ;; The baseline cell sets temporary water column variable
          let baseline-total-height precision ( surface-water-column + elevation ) 4 ;; The baseline cell sets temporary total height variable
          let central-patch self
          ;; Each neighboring cell completes this procedure in order from lowest elevation to highest.
          foreach Neighbor-Elevation-list [
            ask ? [
              ;; Save the local values for itself
              let neighbor-water-column [surface-water-column] of self
              let neighbor-total-height [ precision ( elevation + surface-water-column ) 4 ] of self
              
              ;; Flow will be the amount of water that will move to or from the baseline cell
              let flow 0
              
              ;; The following is the equilibrium level between the two cells.
              let half-difference-height precision ( ( neighbor-total-height - baseline-total-height) / 2 ) 4
              
              ;; For flows:
              ;; Negative means outgoing water
              ;; Positive means incoming water
              
              if half-difference-height != 0 [
                ;; There are ultimately only 4 possibilities for flow calculations, but after going through a decision tree, only one will be used for each neighbor.             
                
                ifelse half-difference-height > 0 [
                  ;; Flow is positive ( potential inflow ) because the neighbor is higher than the baseline cell.  Water will flow to baseline cell.
                  
                  ;; Check whether the neighbor's water column is higher than the equilibrium level
                  ifelse neighbor-water-column >= abs half-difference-height [
                    ;; If the height of water on the neighbor is higher than half the difference between the cell's heights, water will move to the equilibrium level from the neighbor cell
                    set flow precision half-difference-height 4
                  ]
                  [
                    ;; If the height of water on the neighbor cell is lower than the amount of water that would be needed achieve equilibrium, move all water
                    set flow precision neighbor-water-column 4
                  ]
                ] ;; Closes positive flow condition
                [
                  ;; Flow is negative ( potential outflow ) because the neighbor is lower than the baseline cell.  Water will flow from baseline cell.
                  
                  ;; Check whether the baseline water column is higher than the equilibrium level
                  ifelse baseline-water-column >= abs half-difference-height [
                    ;; If the height of water on the baseline cell is higher than have the difference between the cell's heights, water will move to the equilibrium level from the baseline cell
                    set flow precision half-difference-height 4
                  ]
                  [
                    ;; If the height of water on the baseline cell is lower than the amount of water that would be needed achieve equilibrium, move all water
                    set flow precision (- baseline-water-column ) 4
                  ]
                ] ;; Closes negative flow condition
                
                ;; Add the flow amounts to the neighbor and center and update the baseline values
                set surface-water-column precision ( surface-water-column - flow ) 4
                ask central-patch [
                  set surface-water-column precision ( surface-water-column + flow ) 4 ;; This adds the flow to the central patch
                  set baseline-water-column surface-water-column ;; Readjust the baseline water column of the central patch
                  set baseline-total-height precision ( surface-water-column + elevation ) 4 ;; Readjust the total height of the central patch
                ]
              ] ;; closes if half-difference-height != 0 [
              
            ]   ;; closes the ask ? (for neighbors)
          ]         ;;closes the sort loop for the neighbors
          ;; Code to automatically detect error
          if surface-water-column < 0 [ print "error - surface water below 0"]
        ] ;; Closes the loop asking if there is water
      ] ;; closes ask ? of the list
    ] ;; closes  foreach elevation list...   
  ] ;; Closes the repeat
    
  ;;; Check for errors
end

to groundwater-flow
  ;; Asynchronous updating of groundwater head (groundwater top) followed by an update of teh capillary zone
  
  If print-procedure-in-code? [ print "groundwater-flow" ]
  
  ;; Based on Zellner 2007 adaptation of Darcy's Law
  
  ;; Debugging code
  set error-code "groundwater flow"
  
  ;; Patches need to calculate their groundwater water volume. It is not set earlier since GW flow is the only procedure that needs at volume
  ask patches [
    ;; Have the patches setup the buffer layer for the future groundwater water column
    set groundwater-top-FUTURE groundwater-top
;    set groundwater-water-column-FUTURE groundwater-water-column
    ;; The the volume
    set groundwater-water-volume precision ( ( ( groundwater-top - soil-bottom ) * large-pore-specific-yield ) * surface-cell ) 4 ;; height without soil - medium and large poresss
  ]
    
  ;; Now all non-boundary cells exchange water with all of their neighbors. Boundary cells will not initiate the exchange
  ;; This process will update ONLY the groundwater top, surface water, and the groundwater-water-column-FUTURE
  ask patches with [ boundary-cell? = false ] [
    
    ;; Recording baseline values of the center cell doing the updating    
    let center-patch self  ;; record what the patch is that is the baseline
    let center-patch-groundwater-top precision groundwater-top-FUTURE 4
    let center-patch-water-volume groundwater-water-volume
    
    ;; If the groundwater top is at the surface, then we include surface water's water column height in the baseline depth
    if groundwater-top-FUTURE = elevation and surface-water-column > 0 [
      set center-patch-groundwater-top precision ( center-patch-groundwater-top + surface-water-column ) 4
      ;; NOTE: The hydraulic head determines the flow, but the flow only comes from groundwater, so we do not include surface water in the center-patch-water-volume
    ]
    
    if print-flow-tracking-data? [ ;; Debugging code
      print ""
      print ""
      type "center cell = " print self
      print "starting values"
      type  "center-patch-groundwater-top = " print groundwater-top-FUTURE
      type  "center-patch-water-volume = " print center-patch-water-volume
    ]

    ask neighbors [ ;; Start exchanging water with nrandom order
      let flow 0
      let neighbor-patch-groundwater-top precision groundwater-top-FUTURE 4
      let neighbor-patch-water-volume groundwater-water-volume
      
      ;; If the groundwater top is at the surface, then we include surface water's water column height
      if groundwater-top-FUTURE = elevation and surface-water-column > 0 [  
        set neighbor-patch-groundwater-top precision ( neighbor-patch-groundwater-top + surface-water-column ) 4
      ;; NOTE: The hydraulic head determines the flow, but the flow only comes from groundwater, so we do not include surface water in the neighbor-patch-volume
      ]
            
      if print-flow-tracking-data? [ ;; Debugging code
        type "neighbor = " print self
        print "starting values"
        type  "neighbor-patch-groundwater-top = " print neighbor-patch-groundwater-top
        type  "neighbor-patch-volume = " print neighbor-patch-water-volume
      ]
      
      ;;;;; Now calculate flow
      ;; Calculate the difference in heads
      let water-level-difference precision ( neighbor-patch-groundwater-top - center-patch-groundwater-top ) 4 
      ;; Calculate the flow using Darcy's Lay
      
      ;;; TO DO - review flow formula.
      ;;; The water depth of which cell - the center cell or the nrighbor cell? Right now it is using the neighbor cell.

      
      set flow precision ( conductivity * groundwater-specific-yield * ( center-patch-groundwater-top - soil-bottom) * water-level-difference ) 4
      ;; If flow is positive, then flow is incoming from the neighbor, so check that flow is constrained by neighbor patch volume
      if flow > 0 [
        if flow > neighbor-patch-water-volume [
         set flow neighbor-patch-water-volume 
       ]
      ]
      ;; If flow is negative, then flow is leaving to the neighbor, so check that flow is constrained by center patch volume
      if flow < 0 [
        if abs flow > center-patch-water-volume [
         set flow ( - center-patch-water-volume )

        ]
      ]
      
      if print-flow-tracking-data? [ ;; Debugging code
       type  "water-level-difference ( neighbor-patch-groundwater-top - center-patch-groundwater-top ) = " print water-level-difference
       type "flow ( conductivity * groundwater-specific-yield * (groundwater-top-FUTURE - soil-bottom) * water-level-difference ) = " print flow
      ]
      
      ;; Now update the volume and height of water in the neighbor and central cell (note that this applies to either direction of flow, due to signs)
      
      ;; First, update this patch, which is the NEIGHBOR CELL
      ;;;;;;;;;
      
      ;; Subtract the flow from this cell (the neighbor)
      set groundwater-water-volume precision ( groundwater-water-volume - flow ) 4
        
      ;; Set the groundwater top using the new water volume
      set groundwater-top-FUTURE precision ( calculate-water-depth-from-volume groundwater-water-volume soil-bottom large-pore-specific-yield ) 4
      ;; Check to see if the groundwater top is above the elevation
      if groundwater-top-FUTURE > elevation [
        set surface-water-column precision ( surface-water-column + ( ( groundwater-top-FUTURE - elevation ) * large-pore-specific-yield ) ) 4
        set groundwater-top-FUTURE elevation
        set groundwater-water-volume precision ( calculate-volume-from-water-depth groundwater-top-FUTURE soil-bottom large-pore-specific-yield ) 4
      ]
      
      if print-flow-tracking-data? [ ;; Debugging code
        print "Neighbor values after flow"
        type  "neighbor's groundwater-top-FUTURE = " print groundwater-top-FUTURE
        type  "neighbor-volume = " print groundwater-water-volume 
      ]
      
      ;; Second, update CENTER PATCH
      ;;;;;;;;;;
      ask center-patch [ 
        ;; Add the flow to this cell
        set groundwater-water-volume precision ( groundwater-water-volume + flow ) 4
        
        ;; Set the groundwater top using the new water volume
        set groundwater-top-FUTURE precision ( calculate-water-depth-from-volume groundwater-water-volume soil-bottom large-pore-specific-yield ) 4
        ;; Check to see if the groundwater top is above the elevation
        if groundwater-top-FUTURE > elevation [
          set surface-water-column precision ( surface-water-column + ( ( groundwater-top-FUTURE - elevation ) * large-pore-specific-yield ) ) 4
          set groundwater-top-FUTURE elevation
          set groundwater-water-volume precision ( calculate-volume-from-water-depth groundwater-top-FUTURE soil-bottom large-pore-specific-yield ) 4
        ]
        
        ;; Now update the center-patch local variables for next neighbor exchange
        set center-patch-water-volume groundwater-water-volume      
        ;; If the groundwater top is at the surface, then we include any surface water height in the center patch depth
        ifelse groundwater-top-FUTURE = elevation [
          set center-patch-groundwater-top precision ( groundwater-top-FUTURE + surface-water-column ) 4
          ;; NOTE: The hydraulic head determines the flow, but the flow only comes from groundwater, so we do not include surface water in the center-patch-water-volume
        ]
        [
          ;; Otherwise, just use the groundwater
          set center-patch-groundwater-top precision groundwater-top-FUTURE 4
        ]
        
        if print-flow-tracking-data? [ ;; Debugging code
        print "Central patch values after flow"
        type  "center-patch-groundwater-top = " print center-patch-groundwater-top
        type  "center-patch-water-volume = " print center-patch-water-volume
        print "and the non-buffer center patch values (should be the same)"
        type  "groundwater-top-FUTURE = " print groundwater-top-FUTURE
        type  "groundwater-water-volume = " print groundwater-water-volume
        type  "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE
        print ""
      ]
        
      ] ;; CLOSES ask central-patch [ 
      
    ] ;; CLOSES  ask neighbors [                    
  ] ;; CLOSES  ask patches with [ boundary-cell? = false ] [
  
  ;; If the watershed is not self contained, then reset to initial conditions
  ;; This process will update ONLY the groundwater top, surface water, and the groundwater-water-column-FUTURE
  if groundwater-self-contained? = false [ 
    reset-boundary 
  ]
  
   
  
  ;; Next, update the GW water column FUTURE, the capillary zone then rezise the RZ and UZ
  ;; but ONLY if the groundwater-water-column-FUTURE is not the same as the current amount, which indicates there was change
  ask patches with [ precision groundwater-top 4 != precision groundwater-top-FUTURE 4 ] [
;    ask patch 1 1 [ show groundwater-top ]
;; First, update the groundwater water column (inclusive of medium AND large pores now)
    set groundwater-water-column-FUTURE precision ( ( groundwater-top-FUTURE - soil-bottom ) * groundwater-specific-yield ) 4
    Update-soil-horizons-after-groundwater-flow
    set error-code "groundwater flow"
    patch-level-check-for-water-errors
;    ask patch 1 1 [
;      type "after " print groundwater-top
;      print ""
;      ]
  ]
  if print-flow-tracking-data? [
    print ""
    print ""
    ask patches [
      print "AFTER ALL FLOWS..."
      show self
      type "groundwater-top = " print groundwater-top
      type "groundwater-water-volume = " print groundwater-water-volume
      type "groundwater-water-column-FUTURE = " print groundwater-water-column-FUTURE
      print ""
    ]
  ]
  
end

to patch-level-check-for-water-errors
   
  ;; See if the water column in the zones is negative
  if groundwater-water-column < 0 [
    print ""
    type "ticks = " print ticks
    type "procedure = " print error-code
    type self print " has groundwater-water-volume BELOW 0 "
    type "groundwater-water-column = " print groundwater-water-column
  ]
  if upper-zone-water-column < 0 [
    print ""
    type "ticks = " print ticks
    type "procedure = " print error-code
    type self print " has upper-zone-water-column BELOW 0 "
    type "upper-zone-water-column = " print upper-zone-water-column
  ]
  if root-zone-water-column < 0 [
    print ""
    type "ticks = " print ticks
    type "procedure = " print error-code
    type self print " has root-zone-water-column BELOW 0 "
    type "root-zone-water-column = " print upper-zone-water-column
  ]
  if surface-water-column < 0 [
    print ""
    type "ticks = " print ticks
    type "procedure = " print error-code
    type self print " has surface-water-column BELOW 0 "
    type "upper-zone-water-column = " print upper-zone-water-column
  ]
  
  ;; See if the saturation is above 100% for the zones
  let max-GW-water precision ( ( elevation - soil-bottom ) * groundwater-specific-yield ) 4 ;; soil column * specific yield
  if groundwater-water-column > max-GW-water [     
    ifelse precision ( abs ( groundwater-water-column - max-GW-water ) ) 4 > .0001 [
      print ""
      type "ticks = " print ticks
      type "procedure = " print error-code
      type self print " has groundwater-water-column ABOVE MAX "
      type "groundwater-water-column = " print groundwater-water-column
      type "max groundwater-water-column = " print max-GW-water
    ]
    [
      set groundwater-water-column precision max-GW-water 4
    ]
  ]
  let max-UZ-water precision ( ( root-zone-bottom - capillary-top ) * medium-pore-specific-yield ) 4 ;; soil column * specific yield
  if upper-zone-water-column > max-UZ-water [     
    ifelse precision ( abs ( upper-zone-water-column - max-UZ-water ) ) 4 > .0001 [
      print ""
      type "ticks = " print ticks
      type "procedure = " print error-code
      type self print " has upper-zone-water-column ABOVE MAX "
      type "upper-zone-water-column = " print upper-zone-water-column
      type "max upper-zone-water-column = " print max-UZ-water
    ]
    [
      set upper-zone-water-column max-UZ-water
    ]
  ]
  let max-RZ-water precision ( ( elevation - root-zone-bottom ) * medium-pore-specific-yield ) 4 ;; soil column * specific yield
  if root-zone-water-column > max-RZ-water [     
    ifelse precision ( abs ( root-zone-water-column - max-RZ-water ) ) 4 > .0001 [
      print ""
      type "ticks = " print ticks
      type "procedure = " print error-code
      type self print " has root-zone-water-column ABOVE MAX "
      type "root-zone-water-column = " print root-zone-water-column
      type "max root-zone-water-column = " print max-RZ-water
    ]
    [
      set root-zone-water-column max-RZ-water
    ]
  ]
end

to reset-boundary  
  ;; If we are assuming this watershed is connected to the outside world, then cells at the edge of the landscape have their groundwater tops reset to the initial levsls determined during setup
  ;; This also resets the surface water to the initial conditions (0 unless the head was above the elevation)
  
  If print-procedure-in-code? [ print "reset-boundary" ]
  
  ask patches with [ boundary-cell? = true ] [  
    ;; Record previous values to record how much water entered or left via the boundary
    set patch-iteration-boundary-gain 0
    set patch-iteration-boundary-loss 0

    ;; Reset to initial levels set in setup
    let surface-water-column-FUTURE initial-surface-water-column
    set groundwater-top-FUTURE initial-groundwater-top
    set groundwater-water-column-FUTURE initial-groundwater-water-column
    
    ;; This records how much water entered or left the landscape due to flow with the landscape outside of the edge
    ;; The change in the groundwater water column.    
    let groundwater-change-due-to-boundary precision ( ( groundwater-top-FUTURE - groundwater-top ) * groundwater-specific-yield ) 4
    ;; The change in the surface water column
    let surface-water-change-due-to-boundary precision ( surface-water-column-FUTURE - surface-water-column ) 4
    ;; Now update patch-level trackers for gross amounts that have entered and left via the boundary being reset
    ;; A negative value means water went out via the boundary; positive means the water is higher now, so new water came in
    ;; Note: recording increase and decrease seprately rather than simply as a net amount to track error
    if groundwater-change-due-to-boundary > 0 [ set patch-iteration-boundary-gain precision ( patch-iteration-boundary-gain + groundwater-change-due-to-boundary ) 4 ]
    if groundwater-change-due-to-boundary < 0 [ set patch-iteration-boundary-loss precision ( patch-iteration-boundary-loss + abs groundwater-change-due-to-boundary ) 4 ]
    if surface-water-change-due-to-boundary > 0 [ set patch-iteration-boundary-gain precision ( patch-iteration-boundary-gain + surface-water-change-due-to-boundary ) 4 ]
    if surface-water-change-due-to-boundary < 0 [ set patch-iteration-boundary-loss precision ( patch-iteration-boundary-loss + abs surface-water-change-due-to-boundary ) 4 ]
    
    ;; Adopt the surface water column future as the current level
    set surface-water-column surface-water-column-FUTURE
    
    if print-flow-tracking-data? [ ;; Debugging code
      print ""
      print self
      type "boundary cell, so groundwater set to initial level of " print Initial-absolute-water-level
      print ""
    ]
  ]
end

to Update-soil-horizons-after-groundwater-flow
   
  If print-procedure-in-code? [ print "Update-soil-horizons-after-groundwater-flow" ]
   
   if print-flow-tracking-data? [
     print "Update-soil-horizons-after-groundwater-flow"
   ]
    ;; Update the capillary top
    ;; Next, determine the CAPILLARY TOP and then the size of the capillary zone soil column
    ;;; The capillary zone is the 0.8 m of soil above the water table unless the elevation (surface) is closer
    set capillary-top-FUTURE precision ( groundwater-top-FUTURE + 0.8 ) 4
    if capillary-top-FUTURE > elevation [ set capillary-top-FUTURE elevation ] ;; If the capillary top would be higher than the elevation, reset it to be the elevation
    
    ;; Next, determine the future ROOT BOTTOM
    ifelse ( precision ( elevation - capillary-top-FUTURE ) 4 )  > .3 and ( precision ( elevation - capillary-top-FUTURE ) 4 ) > root-length [
      ;; ( elevation - capillary-top ) is the height of the soil column above the capillary top
      ;; As long as the soil column is at least 0.3 and the root does not go into the CZ...
      ;; The future bottom will be the elevation minus whichever is larger, the root length or 0.3 m
      ;; We are subtracting what will be the soil column from the elevation to get the bottom
      ;; Otherwise, the bottom is the the elevation minus the soil height above the capillary zone
      ;; We condensed this into an "and" rather than having the soil-column-above-CZ > root-length be
      ;; a sub condition because the "else" in both cases is set root-zone-soil-column soil-column-above-CZ
      set root-zone-bottom-FUTURE precision ( elevation - ( max (list .3 root-length ) ) ) 4 ;; 
    ]
    [ ;; the else
      ;; (a) soil-column-above-CZ > .3  but soil-column-above-CZ <= root length or (b) soil-column-above-CZ <= .3 (and any root length)
      ;; the soil-column-above-CZ is smaller than the RZ minimum of .3 or the root descends into the CZ
      set root-zone-bottom-FUTURE capillary-top-FUTURE ;; the root bottom will be the same as the capilalry top
    ]  
    
    ;; NOW - Update the root and upper sones
    ;; The groundwater may have gone up or down but cannot be equal because the procedure was called only when there is a change
    ;; If the groundwater went up, then we use the conditions that applied when updating after infiltration: water from the root and/or upper zones descended to the groundwater.
    ;; If the groundwater went down, then we use the conditions that applied after evapotranspiration: water fron the groundwater zone moved to the root and/or upper zones.
    if groundwater-top-FUTURE = groundwater-top [ print "NOTE - look at Update-soil-horizons-after-groundwater-flow procedure - the future and current GW top should not be equal" ]
    ifelse groundwater-top-FUTURE > groundwater-top [
      ;; This is the case when the groundwater went up
      ;; Update the root and upper zone water columns. At least one will be smaller now
      if print-Update-soil-horizons-after-groundwater-flow? [
        print ""
        type self print " had the GW go up after GW flow"  
      ]
      Update-RZ-UZ-after-GW-went-up      
    ] ;; END of first condition for groundwater-water-column-FUTURE > groundwater-water-column [
    [ 
      ;; This is the case when the groundwater went down
      ;; Update the ROOT and UPPER ZONE SOIL COLUMNS. At least one of them will now be LARGER.
      if print-Update-soil-horizons-after-groundwater-flow? [
        print ""
        type self print " had the GW go down after GW flow"  
      ]
      Update-RZ-UZ-after-GW-went-down
    ]  ;; END of the else for ifelse groundwater-water-column-FUTURE > groundwater-water-column [
    
    ;; Adopt the future values as current values
    set groundwater-water-column precision groundwater-water-column-FUTURE 4
    set groundwater-top groundwater-top-FUTURE
    set capillary-top capillary-top-FUTURE
    set root-zone-bottom root-zone-bottom-FUTURE
end

to-report calculate-volume-from-water-depth [wd sd p]
  ;; Determines the volume of the water in the groundwater zone
  let volume precision ( ( (wd - sd) * surface-cell * p) ) 4
  ;; wd is the groundwater-top, or depth in absolute terms to the groundwater
  ;; sd is doil-depth or the depth in absolute terms of the bottom of the soil
  ;; p is porosoty, or groundwater-specific-yield
  ;;type "water-depth imported to calculate volume = " show h
  ;;type "elevation to calculate volume = " show elevation
  ;;type "soil-depth to calculate volume = " show soil-depth
  ;;type "surfCell to calculate volume = " show surfCell
  ;;type "specific-yield to calculate volume = " show specific-yield
  ;;type "volume from calculate-volume-from-water-depth = " show volume
  report volume
end

to-report calculate-water-depth-from-volume [v sd p]
  ;; Determines the depth to the groundwater top (groundwater water depth) using GW water volume
  report precision ( ( (v / ( surface-cell * p ) ) + sd ) ) 4
end


to calculate-globals
  ;; Global variables used in plotting and error tracking
 
  If print-procedure-in-code? [ print "calculate-globals" ]
  
  let sum-rain precision ( count patches * daily-rain-amount ) 4
  let sum-evaporation precision ( sum [ evaporation-amount ] of patches ) 4
  let sum-transpiration precision ( sum [ transpiration-amount ] of patches ) 4
  let sum-boundary-gain precision ( sum [ patch-iteration-boundary-gain ] of patches ) 4
  let sum-boundary-loss precision ( sum [ patch-iteration-boundary-loss ] of patches ) 4
  
  ;;; Update tracking of water in landscape and then calculate the margin of error
  set Landscape-cumulative-net-boundary-gain precision ( Landscape-cumulative-net-boundary-gain + sum-boundary-gain ) 4
  set Landscape-cumulative-net-boundary-loss precision ( Landscape-cumulative-net-boundary-loss + sum-boundary-loss ) 4
  set sum-E precision ( sum-E + sum-evaporation ) 4 ;; total amount that evaporated
  set sum-T precision ( sum-T + sum-transpiration ) 4 ;; total amount that transpired
  set Total-loss-through-ET precision ( sum-E + sum-T ) 4
  ask patches [
    set patch-total-water-height precision ( groundwater-water-column + surface-water-column + upper-zone-water-column + root-zone-water-column + ( (capillary-top - groundwater-top ) * medium-pore-specific-yield ) ) 4
  ]
  let landscape-total-water-height-FUTURE  precision ( sum [ patch-total-water-height ] of patches ) 4
  ;; Error
  let water-in-now precision ( sum-rain + sum-boundary-gain ) 4
  let water-out-now precision ( sum-evaporation + sum-transpiration + sum-boundary-loss ) 4
  set Iteration-discrepancy 100 - ( (
      landscape-total-water-height-FUTURE / precision ( Landscape-current-water-height + water-in-now - water-out-now ) 4
      ) * 100 )
  set Cumulative-discrepancy 100 - ( ( ( Landscape-cumulative-water-height + Landscape-cumulative-net-boundary-gain - Total-loss-through-ET - Landscape-cumulative-net-boundary-loss - Total-loss-through-runoff ) / Landscape-current-water-height ) * 100 )
      
;       landscape-total-water-height + Total-loss-through-ET + Total-loss-through-runoff + Landscape-cumulative-net-boundary-loss ) /  ( total-water-height-created + Landscape-cumulative-net-boundary-gain ) * 100 )

 
 ;; inputs + original - outputs should be equal to there now

 
 
 if print-total-water-heights? [
    ask patch 0 0 [
      print ""
      type "ticks = " print ticks
      type "surface-water-column = " print surface-water-column
      type "root-zone-water-column = " print root-zone-water-column
      type "upper-zone-water-column = " print upper-zone-water-column
      type "CZ-water-column = " print ( (capillary-top - groundwater-top ) * medium-pore-specific-yield )
      type "groundwater-water-column = " print groundwater-water-column
      type "patch-total-water-height = " print patch-total-water-height
      type "root-zone-bottom = " print root-zone-bottom
      type "capillary-top = " print capillary-top
      type "groundwater-top = " print groundwater-top
    ]
    print "Landscape level"
    type "starting landscape-total-water-height = " print Landscape-current-water-height
    type "rain = " print sum-rain
    type "transpiration = " print sum-transpiration
    type "evaporation = " print sum-evaporation
    type "landscape-total-water-height-FUTURE = " print landscape-total-water-height-FUTURE
    type "water-in-now = " print water-in-now
    type "water-out-now = " print water-out-now
    type "denominator = " print precision ( Landscape-current-water-height + water-in-now - water-out-now ) 4
    print "iteration discrepancy is landscape-total-water-height-FUTURE / ( starting height + rain - ET )"
  ]
 
  set Landscape-current-water-height  landscape-total-water-height-FUTURE
end


to do-plots
  ;; TO DO change plot labels to indicate water column, not volume
  ;; Plot data in graphs in the interface
  set-current-plot "Total water height now"
  set-current-plot-pen "Total Water Now"
  plot Landscape-current-water-height
  set-current-plot-pen "Total Water Created"
  plot Landscape-cumulative-water-height
  set-current-plot-pen "Runoff"
  plot   Total-loss-through-runoff
  set-current-plot-pen "Evapotranspiration"
  plot   Total-loss-through-ET
  set-current-plot-pen "Evaporation"
  plot   Sum-E
  set-current-plot-pen "Transpiration"
  plot   Sum-T
  
  set-current-plot "ETc Values"
  set-current-plot-pen "Pasture ETc"
  plot daily-ETo * daily-kcb-pasture
  set-current-plot-pen "Soy ETc"
  plot daily-ETo * daily-Kcb-soy
  set-current-plot-pen "Maize ETc"
  plot daily-ETo * daily-kcb-Maize
  set-current-plot-pen "cover-before-soy ETc"
  plot daily-ETo * daily-kcb-cover-before-soy
  set-current-plot-pen "Wheat-before-soy ETc"
  plot daily-ETo * daily-Kcb-Wheat-before-soy
  set-current-plot-pen "Short-cycle-soy ETc"
  plot daily-ETo * daily-Kcb-short-cycle-soy 
  set-current-plot-pen "ETo"
  plot daily-ETo
  
  set-current-plot "Center Patch Soil Horizons"
  set-current-plot-pen "Groundwater Top"
  ask landscape-center-patch [ plot groundwater-top ]
  set-current-plot-pen "Capillary Top"
  ask landscape-center-patch [ plot capillary-top ]
  set-current-plot-pen "Root Zone Bottom"
  ask landscape-center-patch [ plot root-zone-bottom ]
  
  
  set-current-plot "Evaporation Values"
  set-current-plot-pen "Pasture"
  plot daily-ETo - ( daily-ETo * daily-kcb-pasture )
  set-current-plot-pen "Soy"
  plot daily-ETo - ( daily-ETo * daily-kcb-soy )
  set-current-plot-pen "Maize"
  plot daily-ETo - ( daily-ETo * daily-kcb-maize )
  set-current-plot-pen "Cover-before-soy"
  plot daily-ETo - ( daily-ETo * daily-kcb-Cover-before-soy )
  
  set-current-plot-pen "Wheat-before-soy"
  plot daily-ETo - ( daily-ETo * daily-kcb-Wheat-before-soy )
  set-current-plot-pen "Short-cycle-soy"
  plot daily-ETo - ( daily-ETo * daily-kcb-short-cycle-soy )
  set-current-plot-pen "ETo"
  plot daily-ETo
  
  set-current-plot "Actual Transpiration Values"
  set-current-plot-pen "Sum Pasture Transpiration"
  ifelse any? patches with [ land-cover = "pasture" ] [
    plot sum [ transpiration-amount ] of patches with [ land-cover = "pasture" ]
  ]
  [ plot 0 ]
  set-current-plot-pen "Sum Soy Transpiration"
  ifelse any? patches with [ land-cover = "soy" ] [
    plot sum [ transpiration-amount ] of patches with [ land-cover = "soy" ]
  ]
  [ plot 0 ]
  set-current-plot-pen "Sum Maize Transpiration"
  ifelse any? patches with [ land-cover = "maize" ] [
    plot sum [ transpiration-amount ] of patches with [ land-cover = "maize" ]
  ]
  [ plot 0 ]
  set-current-plot-pen "Sum Cover-before-soy Transpiration"
  ifelse any? patches with [ land-cover = "cover-before-soy" ] [
    plot sum [ transpiration-amount ] of patches with [ land-cover = "cover-before-soy" ]
  ]
  [ plot 0 ]
  set-current-plot-pen "Sum Wheat-before-soy Transpiration"
  ifelse any? patches with [ land-cover = "wheat-before-soy" ] [
    plot sum [ transpiration-amount ] of patches with [ land-cover = "wheat-before-soy" ]
  ]
  [ plot 0 ]
  set-current-plot-pen "Sum Short-cycle-soy Transpiration"
  ifelse any? patches with [ land-cover = "short-cycle-soy" ] [
    plot sum [ transpiration-amount ] of patches with [ land-cover = "short-cycle-soy" ]
  ]
  [ plot 0 ]
  
  set-current-plot "Actual Evaporation Values"
  set-current-plot-pen "Evaporation"
  plot sum [ evaporation-amount ] of patches 
  
  if any? patches with [ transpiration-amount < 0 ] [
    type "ticks = " print ticks
    print "transpiration-amount of at least one patch less than 0"
    ask patches with [ transpiration-amount < 0 ] [
      print self
      print land-cover
      print transpiration-amount
    ]
    print ""
  ]
  if any? patches with [ evaporation-amount < 0 ] [
    print "evaporation-amount of at least one patch less than 0"
    type "ticks = " print ticks
    ask patches with [ evaporation-amount < 0 ] [
      print self
      print land-cover
      print evaporation-amount
    ]
    print ""
  ]

  set-current-plot "KCB"
  set-current-plot-pen "pasture"
  plot daily-kcb-pasture
  set-current-plot-pen "soy"
  plot daily-Kcb-soy
  set-current-plot-pen "maize"
  plot daily-kcb-maize
  set-current-plot-pen "cover-before-soy"
  plot daily-kcb-cover-before-soy
  set-current-plot-pen "Wheat-before-soy"
  plot daily-kcb-Wheat-before-soy
  set-current-plot-pen "Short-cycle-soy"
  plot daily-kcb-short-cycle-soy
  
  set-current-plot "Precipitation"
  set-current-plot-pen "rain"
  plot daily-rain-amount
  
  set-current-plot "ETo"
  set-current-plot-pen "ETo"
  plot daily-ETo
  
  
  set-current-plot "Roots"
  set-current-plot-pen "pasture"
  plot daily-pasture-root-length
  set-current-plot-pen "soy"
  plot daily-soy-root-length
  set-current-plot-pen "maize"
  plot daily-maize-root-length
  set-current-plot-pen "cover-before-soy"
  plot daily-cover-before-soy-root-length
  set-current-plot-pen "Wheat-before-soy"
  plot daily-Wheat-before-soy-root-length
  set-current-plot-pen "Short-cycle-soy"
  plot daily-short-cycle-soy-root-length

end

to export-data
  ;; If the option to export patch level data to a separate file during a run is activated, then this exports the data to the file opened during setup
  ;foreach sort patches [
  ;  ask ? [
  ask patch 0 0 [
    file-write precision ( elevation - groundwater-top ) 5
    file-write surface-water-column
;      ;let variable-list (word pycor "," ticks "," groundwater-top ","))
;      ;file-print (word pxcor "," pycor "," ticks "," groundwater-top ",")
;      file-write pxcor
;      file-write pycor
;      file-write ticks
;      file-write year
;      file-write day
;      file-write last-day-0-decision
      file-write land-cover
;      file-write surface-water-column
;      if export-root-zone-soil-column? [ file-write ( elevation - root-zone-bottom ) ]
;      if export-root-zone-water-column? [ file-write root-zone-water-column ]
;      if export-upper-zone-water-column? [ file-write upper-zone-water-column ]
;      if export-upper-zone-soil-column? [ file-write ( root-zone-bottom - capillary-top ) ]
;      file-write ( elevation - groundwater-top )
;      if export-groundwater-top? [ file-write groundwater-top ]
;      if export-groundwater-water-column?  [ file-write groundwater-water-column ]
;      if export-roots-damaged? [ file-write roots-damaged? ]
;      if export-root-bottom? [ file-write root-bottom ]
;      file-write root-length
;      file-write percent-roots-in-root-zone
;      file-write ETc
;      file-write water-available-root-zone
;      file-write water-available-capillary-zone
;      file-write transpiration-amount
;      file-write evaporation-amount
;      file-write ETc
;      file-write water-deficit
;      file-write cumulative-percent-daily-unmet-need
;      file-write elevation
;      ;     if export-crop-yield-factor? [ file-write crop-yield-factor ]
;      if export-dead? [ file-write dead? ]
;      if export-tonnage-per-hectare?  [
;        file-write soy-tonnage-per-hectare
;        file-write pasture-tonnage-per-hectare
;        file-write cover-before-soy-tonnage-per-hectare
;        file-write wheat-before-soy-tonnage-per-hectare
;        file-write short-cycle-soy-tonnage-per-hectare
;        file-write maize-tonnage-per-hectare
;      ]
;      file-write daily-unmet-need
;      file-write cumulative-percent-daily-unmet-need
      file-write annual-yield
;      file-write years-soy-planted
;      file-write years-pasture-planted
;      file-write years-cover-before-soy-planted
;      file-write years-wheat-before-soy-planted
;      file-write years-short-cycle-soy-planted
;      file-write years-maize-planted
;      file-write patch-soy-output
;      file-write patch-pasture-output
;      file-write patch-cover-before-soy-output
;      file-write patch-wheat-before-soy-output
;      file-write patch-short-cycle-soy-output
;      file-write patch-maize-output
;      
;      if export-boundary-cell?  [ file-write boundary-cell?]
;      file-write daily-rain-amount
;      file-write daily-ETo
;      if export-daily-Kcb-values? [
;        file-write daily-Kcb-soy
;        file-write daily-kcb-pasture
;        file-write daily-kcb-cover-before-soy
;        file-write daily-Kcb-wheat-before-soy
;        file-write daily-Kcb-short-cycle-soy
;        file-write daily-Kcb-maize
;      ]
;      if export-daily-root-lengths? [
;        file-write daily-soy-root-length
;        file-write daily-pasture-root-length
;        file-write daily-cover-before-soy-root-length
;        file-write daily-wheat-before-soy-root-length
;        file-write daily-short-cycle-soy-root-length
;        file-write daily-maize-root-length
;      ]
;      if export-total-water-volume?  [ file-write patch-total-water-height]
;      file-write farmId
      file-print ""
  ]
;    ]
;  ]
end

to reset-year
;; Reset time variables, advance the year counter, and updates the shortened annual list from the larger input files

  set day 0
  set year year + 1
  if year != Years-of-run [ ;; If it is not the end of the run then...
    reset-annual-list ;; Creates a list of the values only for the current year from the larger input file
  ]
  ask patches [ ;; Add the year to the end of the planting history to divide data more easily
    set planting-history lput ( word "year" year ) planting-history
  ]
end
  
to reset-annual-list 
  ;; Creates a list of the values only for the current year from the larger input file
  ;; Different from setup-annual-list because ticks had not previously been set after the setup finished

  If print-procedure-in-code? [ print "reset-annual-list" ]

  ifelse Water-Scenario = "Dry" or Water-Scenario = "Average" or Water-Scenario = "Wet"[
    ;; Just keep repeating the list
    set current-year-values imported-values-list 
  ]
  [
    ;; Otherwise, need to cut a portion of the larger list and determine the number of days in the year
    let starting-item ( ticks * 3 )
        let year-column []
    let year-counter 0
    foreach imported-values-list [
      if ( year-counter - 2 ) mod 3 = 0 [
        set year-column lput ? year-column
      ]
      set year-counter year-counter + 1
    ]
    let current-year-only filter [? = year] year-column
    set days-in-year length current-year-only
    ;; 3 columns in the input file. 3 x days = 1 beyond the number, which is needed since sublist requires going one beyond. It is one beyond already because the lists start at 0 not 1
    set current-year-values sublist imported-values-list starting-item ( starting-item + ( 3 * days-in-year ) ) 
    set days-in-year days-in-year - 1
  ]
end

to calculate-output-totals
  ;; Sum the outputs of each crop for the entire landscape and determine the maximum potential output
  
  If print-procedure-in-code? [ print "calculate-output-totals" ]
  
  ( foreach crop-list [
    ;; Generates: set total-______-output sum [ patch-______-output ] of patches"
    let total-output-sentence ( word "set total-" ? "-output sum [ patch-" ? "-output ] of patches" )
    run total-output-sentence

    ;; Generates "set ______-maximum-output _______-tonnage-per-patch * sum [ years-______-planted ] of patches"    
    let maximum-output-sentence ( word "set " ? "-maximum-output " ? "-tonnage-per-patch * sum [ years-" ? "-planted ] of patches" )
    run maximum-output-sentence
  ] ) 
end
@#$#@#$#@
GRAPHICS-WINDOW
626
10
1200
605
5
5
51.33333333333334
1
10
1
1
1
0
0
0
1
-5
5
-5
5
0
0
1
ticks
30.0

BUTTON
435
197
536
230
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
435
234
536
267
Go (once)
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
999
1324
1521
1539
ETc Values
NIL
NIL
0.0
10.0
0.0
0.0050
true
true
"" ""
PENS
"Pasture ETc" 1.0 0 -955883 true "" ""
"Soy ETc" 1.0 0 -5825686 true "" ""
"Maize ETc" 1.0 0 -4079321 true "" ""
"Cover-before-soy ETc" 1.0 0 -6459832 true "" ""
"Wheat-before-soy ETc" 1.0 0 -12345184 true "" ""
"Short-cycle-soy ETc" 1.0 0 -16777216 true "" ""
"ETo" 1.0 0 -7500403 true "" ""

CHOOSER
28
441
214
486
Water-Scenario
Water-Scenario
"Dry" "Average" "Wet" "Multi-year Pampas data" "5-year-series"
0

BUTTON
435
271
536
304
Go (forever)
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

CHOOSER
435
359
616
404
labeled-variable
labeled-variable
"none" "dead?" "land-cover" "total-water-volume" "groundwater-top + surface water" "surface-water-column" "root-zone-bottom" "root-zone-soil-column" "root-zone-water-column" "upper-zone-water-column" "root-zone-water-column+upper-zone-water-column" "upper-zone-soil-column" "capillary-top" "capillary-water-column" "capillary-soil-column" "soil-column-above-groundwater" "groundwater-top" "groundwater-water-column" "root-bottom" "root-length" "percent-roots-in-upper-zone" "percent-roots-in-capillary-zone" "water-need" "daily-unmet-need" "crop-yield-factor" "annual-yield" "water-available-root-zone" "water-available-capillary-zone" "transpiration-amount" "evaporation-amount" "water-deficit" "top of water" "elevation" "soil-bottom" "days-no-water" "farmId" "cumulative-percent-daily-unmet-need" "days-current-crop-critical"
15

SLIDER
28
307
232
340
Initial-absolute-water-level
Initial-absolute-water-level
-10
11
-2
.25
1
m
HORIZONTAL

SLIDER
27
202
230
235
Initial-saturation-medium-pores
Initial-saturation-medium-pores
0
1
1
.01
1
NIL
HORIZONTAL

BUTTON
436
322
537
355
Label patches
apply-patch-labels
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
1597
555
1977
600
Total water created (height in patch terms - excluding boundary water gain)
Landscape-cumulative-water-height
17
1
11

MONITOR
1599
748
1845
793
Total water now
Landscape-current-water-height
17
1
11

MONITOR
1598
652
1844
697
Total water lost through ET
Total-loss-through-ET
17
1
11

PLOT
1602
851
1956
1001
Total water height now
Time
m
0.0
364.0
0.0
10.0
true
true
"" ""
PENS
"Total Water Now" 1.0 0 -16777216 true "" ""
"Total Water Created" 1.0 0 -7500403 true "" ""
"Runoff" 1.0 0 -2674135 true "" ""
"Evapotranspiration" 1.0 0 -955883 true "" ""
"Evaporation" 1.0 0 -5825686 true "" ""
"Transpiration" 1.0 0 -1184463 true "" ""

MONITOR
1598
604
1816
649
Runoff that left the landscape
Total-loss-through-runoff
17
1
11

PLOT
513
1114
936
1264
KCB
NIL
NIL
0.0
10.0
0.0
1.2
true
true
"" ""
PENS
"Pasture" 1.0 0 -955883 true "" ""
"Soy" 1.0 0 -5825686 true "" ""
"Maize" 1.0 0 -4079321 true "" ""
"Cover-before-soy" 1.0 0 -6459832 true "" ""
"Wheat-before-soy" 1.0 0 -11221820 true "" ""
"Short-cycle-soy" 1.0 0 -16777216 true "" ""

PLOT
493
1267
865
1417
ETo
NIL
NIL
0.0
10.0
0.0
0.0050
true
true
"" ""
PENS
"ETo" 1.0 0 -16777216 true "" ""

PLOT
37
1432
467
1593
Roots
NIL
NIL
0.0
10.0
0.0
4.0
true
true
"" ""
PENS
"soy" 1.0 0 -955883 true "" ""
"pasture" 1.0 0 -5825686 true "" ""
"maize" 1.0 0 -4079321 true "" ""
"Cover-before-soy" 1.0 0 -6459832 true "" ""
"Wheat-before-soy" 1.0 0 -11221820 true "" ""
"Short-cycle-soy" 1.0 0 -16777216 true "" ""

PLOT
29
1112
399
1262
Precipitation
NIL
NIL
0.0
10.0
0.0
0.15
true
true
"" ""
PENS
"rain" 1.0 0 -16777216 true "" ""

PLOT
998
1108
1538
1321
Actual Transpiration Values
NIL
NIL
0.0
10.0
0.0
0.65
true
true
"" ""
PENS
"Sum Pasture Transpiration" 1.0 0 -955883 true "" ""
"Sum Soy Transpiration" 1.0 0 -5825686 true "" ""
"Sum Maize Transpiration" 1.0 0 -4079321 true "" ""
"Sum Cover-before-soy Transpiration" 1.0 0 -6459832 true "" ""
"Sum Wheat-before-soy Transpiration" 1.0 0 -12345184 true "" ""
"Sum Short-cycle-soy Transpiration" 1.0 0 -16777216 true "" ""

SLIDER
267
237
408
270
Slope
Slope
0
5
0
.25
1
NIL
HORIZONTAL

SLIDER
269
343
410
376
Days-no-water-death
Days-no-water-death
0
10
7
1
1
NIL
HORIZONTAL

SWITCH
28
487
146
520
Fixed-runoff?
Fixed-runoff?
0
1
-1000

SLIDER
28
521
179
554
Fixed-runoff-amount
Fixed-runoff-amount
0
72
30
1
1
%
HORIZONTAL

MONITOR
640
749
695
786
Crop years
sum [ years-soy-planted ] of patches
4
1
9

MONITOR
639
692
694
729
Crop years
sum [ years-pasture-planted ] of patches
1
1
9

MONITOR
898
745
959
786
Output
total-soy-output
2
1
10

MONITOR
898
688
958
729
Output
total-pasture-output
2
1
10

SLIDER
259
605
413
638
%-Soy
%-Soy
0
100
33
1
1
%
HORIZONTAL

SWITCH
28
27
156
60
Export-data?
Export-data?
1
1
-1000

SWITCH
26
132
212
165
groundwater-self-contained?
groundwater-self-contained?
0
1
-1000

MONITOR
1599
796
1826
841
Total water entered through boundary
Landscape-cumulative-net-boundary-gain
17
1
11

MONITOR
1598
700
1800
745
Total water left through boundary
Landscape-cumulative-net-boundary-loss
17
1
11

SLIDER
27
237
211
270
groundwater-specific-yield
groundwater-specific-yield
0
1
0.31
.01
1
NIL
HORIZONTAL

SLIDER
27
272
211
305
medium-pore-specific-yield
medium-pore-specific-yield
.01
1
0.12
.01
1
NIL
HORIZONTAL

SWITCH
1592
397
1814
430
print-evaporation-tracking-data?
print-evaporation-tracking-data?
1
1
-1000

SWITCH
1936
68
2039
101
no-rain?
no-rain?
1
1
-1000

SLIDER
1753
106
1930
139
groundwater-outlier-value
groundwater-outlier-value
-10
11
0
.5
1
NIL
HORIZONTAL

SWITCH
1588
106
1747
139
groundwater-outlier?
groundwater-outlier?
1
1
-1000

SWITCH
1937
105
2099
138
no-evapotranspiration?
no-evapotranspiration?
1
1
-1000

SWITCH
2023
339
2194
372
print-flow-tracking-data?
print-flow-tracking-data?
1
1
-1000

SWITCH
1589
67
1747
100
surface-water-outlier?
surface-water-outlier?
1
1
-1000

SLIDER
1754
67
1929
100
surface-water-outlier-value
surface-water-outlier-value
0
10
0
.1
1
NIL
HORIZONTAL

SWITCH
1592
432
1814
465
print-water-available-tracking-data?
print-water-available-tracking-data?
1
1
-1000

SWITCH
1828
357
1998
390
print-update-RZ-UZ?
print-update-RZ-UZ?
1
1
-1000

SWITCH
1592
325
1764
358
print-rain-infiltration?
print-rain-infiltration?
1
1
-1000

SWITCH
2020
265
2206
298
print-water-correction-data?
print-water-correction-data?
1
1
-1000

MONITOR
784
749
897
786
Maximum possible output
soy-maximum-output
4
1
9

MONITOR
784
692
897
729
Maximum possible output
pasture-maximum-output
4
1
9

SWITCH
266
168
409
201
Import-landscape?
Import-landscape?
1
1
-1000

SWITCH
1594
1069
1822
1102
export-root-zone-soil-column?
export-root-zone-soil-column?
1
1
-1000

SWITCH
1594
1106
1822
1139
export-root-zone-water-column?
export-root-zone-water-column?
1
1
-1000

SWITCH
1593
1144
1822
1177
export-upper-zone-water-column?
export-upper-zone-water-column?
1
1
-1000

SWITCH
1593
1180
1822
1213
export-upper-zone-soil-column?
export-upper-zone-soil-column?
1
1
-1000

SWITCH
1594
1216
1822
1249
export-groundwater-top?
export-groundwater-top?
1
1
-1000

SWITCH
1596
1252
1823
1285
export-groundwater-water-column?
export-groundwater-water-column?
1
1
-1000

SWITCH
1594
1287
1823
1320
export-roots-damaged?
export-roots-damaged?
1
1
-1000

SWITCH
1859
1252
2045
1285
export-root-bottom?
export-root-bottom?
1
1
-1000

SWITCH
1860
1287
2046
1320
export-dead?
export-dead?
1
1
-1000

SWITCH
1856
1068
2045
1101
export-tonnage-per-hectare?
export-tonnage-per-hectare?
1
1
-1000

SWITCH
1857
1107
2045
1140
export-boundary-cell?
export-boundary-cell?
1
1
-1000

SWITCH
1858
1142
2046
1175
export-daily-Kcb-values?
export-daily-Kcb-values?
1
1
-1000

SWITCH
1859
1218
2044
1251
export-total-water-volume?
export-total-water-volume?
1
1
-1000

SLIDER
266
202
408
235
Landscape-size
Landscape-size
1
20
5
1
1
NIL
HORIZONTAL

SLIDER
267
271
408
304
Patch-dimensions
Patch-dimensions
15
200
100
5
1
m
HORIZONTAL

SLIDER
26
167
212
200
conductivity
conductivity
0
10
2
1
1
NIL
HORIZONTAL

SLIDER
265
102
406
135
Years-of-run
Years-of-run
1
40
2
1
1
years
HORIZONTAL

SLIDER
417
606
572
639
%-Cover-Crop
%-Cover-Crop
0
100
33
1
1
NIL
HORIZONTAL

CHOOSER
270
389
412
434
Planting-scenario
Planting-scenario
"0" "1a" "1b" "1c" "1d" "1e" "1f" "Observed Sequence" "Strict Rotation" "Individual Adaptation"
1

MONITOR
546
253
603
298
Day
day
17
1
11

MONITOR
639
867
695
904
Crop years
sum [ years-cover-before-soy-planted ] of patches
4
1
9

MONITOR
899
863
961
904
Output
total-cover-before-soy-output
4
1
10

MONITOR
783
867
898
904
Maximum possible output
cover-before-soy-maximum-output
4
1
9

MONITOR
545
200
602
245
Ticks
ticks
17
1
11

SWITCH
1827
290
1985
323
print-unmet-need?
print-unmet-need?
1
1
-1000

SWITCH
1592
362
1754
395
print-transpiration?
print-transpiration?
1
1
-1000

SWITCH
1592
468
1712
501
print-roots?
print-roots?
1
1
-1000

SWITCH
1859
1181
2045
1214
export-daily-root-lengths?
export-daily-root-lengths?
1
1
-1000

MONITOR
547
307
604
352
Year
year
17
1
11

TEXTBOX
273
586
376
604
Scenario \"0\" only
11
0.0
1

TEXTBOX
1587
44
2027
64
Controls to manually override processes; useful for testing
13
0.0
1

TEXTBOX
1592
224
2090
245
If enabled, will print out lots of data showing calculations for this portion of code
13
0.0
1

TEXTBOX
167
10
431
87
Activate before hitting setup; if activated, a large amount of data will be exported, which may result in a very large output file size. Scroll down to the bottom right of the interface for a list of optional variables to export. If in error, Export-data? is toggled and then the model is initialized, close and reopen the model in order to close the output file created by initialization
9
0.0
1

SWITCH
1936
142
2063
175
ONEcm-rain?
ONEcm-rain?
1
1
-1000

SWITCH
27
347
206
380
no-groundwater-flow?
no-groundwater-flow?
1
1
-1000

SWITCH
2021
302
2170
335
print-tracking-error?
print-tracking-error?
1
1
-1000

MONITOR
960
749
1025
786
% Maximum
( total-soy-output / soy-maximum-output ) * 100
2
1
9

MONITOR
959
692
1024
729
% Maximum
( total-pasture-output / pasture-maximum-output ) * 100
2
1
9

MONITOR
962
867
1025
904
% Maximum
( total-cover-before-soy-output / cover-before-soy-maximum-output ) * 100
2
1
9

PLOT
16
1264
472
1428
Evaporation Values
NIL
NIL
0.0
10.0
0.0
0.0050
true
true
"" ""
PENS
"Pasture" 1.0 0 -955883 true "" ""
"Soy" 1.0 0 -5825686 true "" ""
"Maize" 1.0 0 -1184463 true "" ""
"Cover-before-soy" 1.0 0 -6459832 true "" ""
"Wheat-before-soy" 1.0 0 -11221820 true "" ""
"Short-cycle-soy" 1.0 0 -16777216 true "" ""
"ETo" 1.0 0 -7500403 true "" ""

MONITOR
641
808
695
845
Crop years
sum [ years-maize-planted ] of patches
4
1
9

MONITOR
784
808
898
845
Maximum possible output
maize-maximum-output
4
1
9

MONITOR
899
804
960
845
Output
total-maize-output
2
1
10

MONITOR
961
808
1025
845
% Maximum
( total-maize-output / maize-maximum-output ) * 100
2
1
9

MONITOR
638
926
695
963
Crop years
sum [ years-Wheat-before-soy-planted ] of patches
4
1
9

MONITOR
782
926
897
963
Maximum possible output
Wheat-before-soy-maximum-output
4
1
9

MONITOR
898
922
961
963
Output
total-wheat-before-soy-output
2
1
10

MONITOR
962
926
1026
963
% Maximum
( total-Wheat-before-soy-output / Wheat-before-soy-maximum-output ) * 100
2
1
9

MONITOR
637
986
694
1023
Crop years
sum [ years-short-cycle-soy-planted ] of patches
4
1
9

MONITOR
783
986
897
1023
Maximum possible output
short-cycle-soy-maximum-output
4
1
9

MONITOR
898
982
963
1023
Output
total-short-cycle-soy-output
2
1
10

MONITOR
964
986
1027
1023
% Maximum
( total-short-cycle-soy-output / short-cycle-soy-maximum-output ) * 100
2
1
9

MONITOR
1220
185
1298
230
Rotation Year
[ Rotation-year ] of landscape-center-patch
17
1
11

SWITCH
1827
255
2011
288
print-planting-decision?
print-planting-decision?
1
1
-1000

OUTPUT
1219
555
1459
736
12

SLIDER
22
624
175
657
May-Dec-ST
May-Dec-ST
.1
3
0.7
.1
1
m
HORIZONTAL

SLIDER
21
661
176
694
May-DT
May-DT
.2
5
2.5
.1
1
m
HORIZONTAL

TEXTBOX
34
604
187
622
Individual adaptation only
12
0.0
1

MONITOR
696
749
783
786
Tonnage per patch
soy-tonnage-per-patch
4
1
9

MONITOR
695
692
783
729
Tonnage per patch
pasture-tonnage-per-patch
4
1
9

MONITOR
696
808
783
845
Tonnage per patch
maize-tonnage-per-patch
4
1
9

MONITOR
696
867
782
904
Tonnage per patch
cover-before-soy-tonnage-per-patch
4
1
9

MONITOR
696
926
781
963
Tonnage per patch
wheat-before-soy-tonnage-per-patch
4
1
9

MONITOR
695
986
782
1023
Tonnage per patch
short-cycle-soy-tonnage-per-patch
4
1
9

TEXTBOX
1221
10
1470
50
Landscape Center Patch Values
16
0.0
1

MONITOR
1348
42
1415
87
Land Cover
[ land-cover ] of landscape-center-patch
17
1
11

MONITOR
1220
137
1330
182
Groundwater Top
[ groundwater-top ] of landscape-center-patch
8
1
11

MONITOR
1220
89
1284
134
Elevation
[ elevation ] of landscape-center-patch
8
1
11

MONITOR
1219
41
1341
86
Surface Water Column
[ surface-water-column ] of landscape-center-patch
8
1
11

MONITOR
1365
90
1415
135
Dead?
[ dead? ] of landscape-center-patch
17
1
11

MONITOR
1342
138
1415
183
Root Length
[ root-length ] of landscape-center-patch
8
1
11

MONITOR
1338
185
1410
230
Harvest Day
[ day-harvest ] of landscape-center-patch
17
1
11

MONITOR
1220
232
1333
277
Day to Check the WT
[ day-check-WTD ] of landscape-center-patch
8
1
11

MONITOR
1339
232
1410
277
Annual Yield
[ annual-yield ] of landscape-center-patch
8
1
11

TEXTBOX
222
461
332
563
                Scenario 0:\n                Scenario 1:\n\n          Strict Rotation:\n Observed Sequence:\n\nIndividual Adaptation:\n
11
0.0
1

TEXTBOX
333
461
609
567
Old model version (no adaptation)\nAll (a) soy, (b) pasture, (c)  maize, (d) cover then soy,\n     (e) wheat then soy, (f) fallow\nStrict observance of maize; soy; wheat-soy rotation\n5-year cycle of wheat-soy; maize; soy; wheat-soy;\n     maize\nUses adaptive decision making\n
11
0.0
1

TEXTBOX
448
32
612
186
     Grey\n      Red\n Orange\n   Brown\n   Yellow\nSky blue\n     Violet\nDarker shade indicates stressed\nVery dark shade indicates dead\nBlue rectangle indicates\n     surface water\n
11
0.0
1

TEXTBOX
491
32
631
130
= fallow\n= pasture\n= regular soy\n= maize\n= cover\n= wheat\n= short cycle soy
11
0.0
1

TEXTBOX
641
676
681
694
Pasture
11
0.0
1

TEXTBOX
641
733
667
751
Soy
11
0.0
1

TEXTBOX
642
793
792
811
Maize
11
0.0
1

TEXTBOX
641
851
791
869
Cover
11
0.0
1

TEXTBOX
639
910
789
928
Wheat
11
0.0
1

TEXTBOX
1600
532
1700
550
Water Balance
12
0.0
1

TEXTBOX
639
970
789
988
Short cycle soy
11
0.0
1

TEXTBOX
104
115
141
133
Water
12
0.0
1

TEXTBOX
83
419
154
437
Precipitation
12
0.0
1

TEXTBOX
312
153
380
171
Landscape
12
0.0
1

TEXTBOX
320
87
356
105
Time
12
0.0
1

TEXTBOX
323
326
356
344
Plants
12
0.0
1

TEXTBOX
66
10
158
28
Exporting data
12
0.0
1

TEXTBOX
263
565
413
583
Planting scenario controols
12
0.0
1

TEXTBOX
458
10
579
44
Landscape key
14
0.0
1

TEXTBOX
637
643
787
663
Crop Yields
16
0.0
1

TEXTBOX
1594
1032
1782
1052
Optional exported values
16
0.0
1

TEXTBOX
1220
522
1469
552
Center patch planting history when using adaptive rules
12
0.0
1

TEXTBOX
1587
10
1737
30
Debugging tools
16
0.0
1

PLOT
1217
289
1570
504
Center Patch Soil Horizons
NIL
NIL
0.0
10.0
-3.0
0.0
true
true
"" ""
PENS
"Groundwater Top" 1.0 0 -16777216 true "" ""
"Capillary Top" 1.0 0 -7858858 true "" ""
"Root Zone Bottom" 1.0 0 -14439633 true "" ""

PLOT
1002
1541
1342
1754
Actual Evaporation Values
NIL
NIL
0.0
10.0
0.0
0.65
true
false
"" ""
PENS
"Evaporation" 1.0 0 -16777216 true "" ""

SWITCH
1828
325
2009
358
plant-death-message?
plant-death-message?
1
1
-1000

SWITCH
1830
497
2108
530
print-Update-RZ-UZ-after-GW-went-up?
print-Update-RZ-UZ-after-GW-went-up?
1
1
-1000

SWITCH
1829
463
2123
496
Print-Update-RZ-UZ-after-GW-went-down?
Print-Update-RZ-UZ-after-GW-went-down?
1
1
-1000

SWITCH
1937
181
2088
214
no-plant-growth?
no-plant-growth?
1
1
-1000

SWITCH
1829
427
2137
460
print-Update-soil-horizons-after-groundwater-flow?
print-Update-soil-horizons-after-groundwater-flow?
1
1
-1000

SWITCH
1590
290
1783
323
print-procedure-in-code?
print-procedure-in-code?
1
1
-1000

SWITCH
1590
256
1788
289
print-total-water-heights?
print-total-water-heights?
1
1
-1000

SWITCH
1829
392
2136
425
Print-Update-soil-horizons-after-evapotranspiration?
Print-Update-soil-horizons-after-evapotranspiration?
1
1
-1000

SLIDER
21
697
175
730
Sep-T
Sep-T
0
2
1.4
.1
1
NIL
HORIZONTAL

SLIDER
23
734
175
767
Oct-T
Oct-T
0
2
1.2
.1
1
NIL
HORIZONTAL

TEXTBOX
187
664
384
776
Key to sliders at left:\nMay/Sep/Oct/Dec = month value used\nST = Shallow Threshold\nDT = Deep Threshold\nT = Threshold (no shallow/deep)
11
0.0
1

TEXTBOX
36
1063
436
1103
Daily Values (based on input files, may vary for patches due to local conditions)
16
0.0
1

TEXTBOX
1005
1069
1219
1109
Actual Values (aggregated)
16
0.0
1

TEXTBOX
271
442
421
460
Guide to Planting-scenario
12
0.0
1

TEXTBOX
446
410
621
438
This sets the variable that is labeled on the landscape
11
0.0
1

SLIDER
26
79
198
112
Export-start-year
Export-start-year
0
30
0
1
1
NIL
HORIZONTAL

@#$#@#$#@
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

puddle
false
0
Rectangle -13345367 true false 15 120 285 210

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experimenttest" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>count patches</metric>
    <enumeratedValueSet variable="medium-pore-specific-yield">
      <value value="0.12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="conductivity">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pore-type-used">
      <value value="&quot;large&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Patch-dimensions">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Days-no-water-death">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Fixed-runoff?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%-Soy">
      <value value="33"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%-Cover-Crop">
      <value value="33"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="plant-death-message?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-outlier?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="surface-water-outlier?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-outlier-value">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="surface-water-outlier-value">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="labeled-variable">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-upper-zone-soil-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-tonnage-per-hectare?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-upper-zone-water-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-daily-root-lengths?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Export-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-root-zone-soil-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-groundwater-top?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-root-zone-water-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-roots-damaged?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-daily-Kcb-values?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-groundwater-water-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-boundary-cell?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-root-bottom?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-total-water-volume?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-dead?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-unmet-need?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-Update-RZ-UZ-after-GW-went-up?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Print-Update-soil-horizons-after-evapotranspiration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-water-correction-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-Update-soil-horizons-after-groundwater-flow?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-tracking-error?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-transpiration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-procedure-in-code?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-planting-decision?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-water-available-tracking-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-roots?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-rain-infiltration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-evaporation-tracking-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Planting-scenario">
      <value value="&quot;1a&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Print-Update-RZ-UZ-after-GW-went-down?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-update-RZ-UZ?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-flow-tracking-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-total-water-heights?">
      <value value="false"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment1111" repetitions="1" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>count turtles</metric>
    <enumeratedValueSet variable="Water-Scenario">
      <value value="&quot;Dry&quot;"/>
      <value value="&quot;Average&quot;"/>
      <value value="&quot;Wet&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-water-level">
      <value value="-0.5"/>
      <value value="-2"/>
      <value value="-3.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Years-of-run">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-groundwater-flow?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Slope">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Fixed-runoff-amount">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Planting-scenario">
      <value value="&quot;1a&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Landscape-size">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Export-data?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-self-contained?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Proactive-deep-threshold">
      <value value="1.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Proactive-shallow-threshold">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Reactive-deep-threshold">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Reactive-shallow-threshold">
      <value value="0.3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Fixed-runoff?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Patch-dimensions">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-evapotranspiration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-plant-growth?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-rain?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ONEcm-rain?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Import-landscape?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-saturation-medium-pores">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Days-no-water-death">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="conductivity">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="medium-pore-specific-yield">
      <value value="0.12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-specific-yield">
      <value value="0.31"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pore-type-used">
      <value value="&quot;large&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%-Cover-Crop">
      <value value="33"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%-proactive">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%-Soy">
      <value value="33"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="surface-water-outlier?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="surface-water-outlier-value">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-outlier?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-outlier-value">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="labeled-variable">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="plant-death-message?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-boundary-cell?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-daily-Kcb-values?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-daily-root-lengths?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-dead?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-groundwater-top?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-groundwater-water-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-root-bottom?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-roots-damaged?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-root-zone-soil-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-root-zone-water-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-tonnage-per-hectare?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-total-water-volume?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-upper-zone-soil-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-upper-zone-water-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-evaporation-tracking-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-flow-tracking-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-planting-decision?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-procedure-in-code?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-rain-infiltration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-roots?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-total-water-heights?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-tracking-error?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-transpiration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-unmet-need?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-update-RZ-UZ?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Print-Update-RZ-UZ-after-GW-went-down?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-Update-RZ-UZ-after-GW-went-up?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Print-Update-soil-horizons-after-evapotranspiration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-Update-soil-horizons-after-groundwater-flow?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-water-available-tracking-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-water-correction-data?">
      <value value="false"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="WTD-climate-landcover" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <enumeratedValueSet variable="Planting-scenario">
      <value value="&quot;1a&quot;"/>
      <value value="&quot;1b&quot;"/>
      <value value="&quot;1c&quot;"/>
      <value value="&quot;1d&quot;"/>
      <value value="&quot;1e&quot;"/>
      <value value="&quot;1f&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Water-Scenario">
      <value value="&quot;Dry&quot;"/>
      <value value="&quot;Average&quot;"/>
      <value value="&quot;Wet&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-absolute-water-level">
      <value value="-5"/>
      <value value="-2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Export-start-year">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Years-of-run">
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-Update-soil-horizons-after-groundwater-flow?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-evapotranspiration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-upper-zone-soil-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-plant-growth?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-tracking-error?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-saturation-medium-pores">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-roots?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Print-Update-soil-horizons-after-evapotranspiration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="May-DT">
      <value value="2.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-water-correction-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%-Cover-Crop">
      <value value="33"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Export-data?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-dead?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-flow-tracking-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Sep-T">
      <value value="1.4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-procedure-in-code?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Days-no-water-death">
      <value value="7"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-daily-Kcb-values?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-unmet-need?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-tonnage-per-hectare?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-evaporation-tracking-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Oct-T">
      <value value="1.2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ONEcm-rain?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-daily-root-lengths?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-outlier-value">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-root-zone-water-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-specific-yield">
      <value value="0.31"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-total-water-heights?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-self-contained?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="medium-pore-specific-yield">
      <value value="0.12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="%-Soy">
      <value value="33"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-groundwater-water-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-groundwater-flow?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-roots-damaged?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-root-bottom?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="May-Dec-ST">
      <value value="0.7"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Landscape-size">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="surface-water-outlier-value">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="plant-death-message?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="surface-water-outlier?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Import-landscape?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-root-zone-soil-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-upper-zone-water-column?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groundwater-outlier?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-boundary-cell?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Patch-dimensions">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="labeled-variable">
      <value value="&quot;soil-column-above-groundwater&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Fixed-runoff?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-planting-decision?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Fixed-runoff-amount">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-Update-RZ-UZ-after-GW-went-up?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Slope">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Print-Update-RZ-UZ-after-GW-went-down?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-update-RZ-UZ?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-rain?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-total-water-volume?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="conductivity">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-rain-infiltration?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-water-available-tracking-data?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="export-groundwater-top?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="print-transpiration?">
      <value value="false"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
